from __future__ import print_function

'''
Tools to analyse the local many-body density matrix (multiplet structure).
'''

try:
    from builtins import range
except:
    pass

import numpy as np
from scipy.linalg import logm
import h5py


def get_rho_histogram(rho, S=None, L=None, J=None, num_ev=0, Rpr_list=None):
    '''
    Get the histogram of the local density matrix with labels.

    This function diagonalizes the reduced local many-body density matrix rho,
    in the order of valence block.
    The label of the resultant eigen-space, such as (averaged) J,
    character chi of the irreducible representation and degeneracy,
    will be computed.
    The final results are ordered accordig to descending order of
    the eigen-values of the eigen-spaces.

    Parameters
    ----------
    rho : 2d array
        reduced local many-body density matrix in sparse matrix format.
    J : csc_matrix
        Total angular momentum :math:`J^{2}` operator in sparse matrix format.
    num_ev : integer
        Number of significant eigen-vectors of rho to be calculated.
    Rpr_list : list.
        List of rotation operations in the local Hilbert space of difference
        valence block.

    Returns
    -------
    vals : array
        Eigen-values of the eigen-spaces of rho.
    j_label : array
        Averaged J values of the eigen-spaces of rho.
    chi_label : array
        Characters of the eigen-spaces of rho as the irreducible
        representation of the rotation group defined by Rpr_list.
    multiplet_degeneracies : array
        Degeneracies of the eigen-spaces of rho.

    '''
    print(" Get rho histogram.")
    if num_ev > rho.shape[0]:
        num_ev = 0
    if num_ev <= 0:
        vals, vecs = np.linalg.eigh(rho)
        vecs = vecs.T
    else:
        from scipy.sparse.linalg import eigsh
        vals, vecs = eigsh(rho, num_ev, which='LM')
        vecs = vecs.T
    # Make sure eigen-values properly ordered.
    idx = vals.argsort()[::-1]
    vals = vals[idx]
    vecs = vecs[idx, :]

    from pygrisb.mbody.local_operator_factory import get_label_list
    print(" Get labels.")
    s_label = get_label_list(S, vecs)
    if s_label is not None:
        s_label = np.sqrt(s_label + 0.25) - 0.5

    l_label = get_label_list(L, vecs)
    if l_label is not None:
        l_label = np.sqrt(l_label + 0.25) - 0.5

    j_label = get_label_list(J, vecs)
    if j_label is not None:
        j_label = np.sqrt(j_label + 0.25) - 0.5

    from pygrisb.math.matrix_util import set_eigen_space, shrink_label
    idx = set_eigen_space(vals)
    vals = shrink_label(vals, idx, method="sum")
    s_label = shrink_label(s_label, idx, method="average")
    l_label = shrink_label(l_label, idx, method="average")
    j_label = shrink_label(j_label, idx, method="average")
    multiplet_degeneracies = np.array(
        [idx[i + 1] - idx[i] for i in range(len(idx) - 1)])

    if Rpr_list is not None:
        # check commutation
        from pygrisb.symm.atom_symm import check_commute_G
        check_commute_G(rho, Rpr_list)

        from pygrisb.symm.atom_symm import get_characters_espace, \
                check_sum_chi2_1
        chi_label, _ = get_characters_espace(Rpr_list, vecs)
        check = check_sum_chi2_1(chi_label)
        if check != "OK":
            print(" Warning: chi-sum error!")
    else:
        chi_label = None

    return vals, s_label, l_label, j_label, chi_label, multiplet_degeneracies


def get_ordered_labels(val_list, n_list, s_list, l_list, j_list, \
        chi_list, d_list):
    '''
    Sort according to eigen-values of rho.
    '''

    idx = np.asarray(val_list).argsort()[::-1]
    val_list = np.asarray(val_list)[idx]
    n_list = np.asarray(n_list)[idx]
    if s_list is not None:
        s_list = np.asarray(s_list)[idx]
    if l_list is not None:
        l_list = np.asarray(l_list)[idx]
    if j_list is not None:
        j_list = np.asarray(j_list)[idx]
    if chi_list != []:
        chi_list = np.asarray(chi_list)[idx]
    d_list = np.asarray(d_list)[idx]
    return val_list, n_list, s_list, l_list, j_list, chi_list, d_list


def get_local_histogram(imp, op_list=["S2","L2","J2"], num_ev=0,
        Rpr_list=None, n_bot = None, n_top = None):
    '''
    Get the local histogram with labels.
    '''

    with h5py.File('EMBED_HAMIL_{}.h5'.format(imp), 'r') as f:
        _nbot = f['/nval_bot'][0]
        _ntop = f['/nval_top'][0]
    if n_bot is None:
        n_bot = _nbot
    if n_top is None:
        n_top = _ntop
    n_bot = max(n_bot, _nbot)
    n_top = min(n_top, _ntop)
    print(' n_bot = {}, n_top = {}'.format(n_bot, n_top))

    val_list = []
    s_list = []
    l_list = []
    j_list = []
    n_list = []
    chi_list = []
    d_list = []

    if "S2" in op_list:
        eval_s2 = 0
    else:
        eval_s2 = None
    if "L2" in op_list:
        eval_l2 = 0
    else:
        eval_l2 = None
    if "J2" in op_list:
        eval_j2 = 0
    else:
        eval_j2 = None
    ent_entropy = 0

    from pygrisb.mbody.local_operator_factory import get_local_operators
    for ival in range(n_bot, n_top+1):
        print(" Valence = {}".format(ival))
        # Get reduced density matrix.
        with h5py.File('EMBED_HAMIL_ANALYSIS_{}.h5'.format(imp), 'r') as f:
            path = '/valence_block_{}/RHO'.format(ival)
            if path not in f:
                continue
            rho = f[path][...].T

        tr_rho = np.trace(rho)
        print(' trace(rho) = {}'.format(tr_rho))

        # entanglement entropy
        if tr_rho > 1.e-6:
            ent_entropy -= np.trace(rho.dot(logm(rho)))

        print(" Get local operators (J2, ...)")
        op_dict = get_local_operators(imp, ival, op_list)

        if "S2" in op_list:
            S = op_dict["S2"]
            eval_s2 += np.trace(S.dot(rho))
        else:
            S = None

        if "L2" in op_list:
            L = op_dict["L2"]
            eval_l2 += np.trace(L.dot(rho))
        else:
            L = None

        if "J2" in op_list:
            J = op_dict["J2"]
            eval_j2 += np.trace(J.dot(rho))
        else:
            J = None

        if Rpr_list is not None:
            Rpr_list = Rpr_list[n_bot : n_top + 1]

        vals, s_label, l_label, j_label, chi_label, degeneracies = \
                get_rho_histogram(rho, S=S, L=L, J=J, num_ev=num_ev, \
                Rpr_list=Rpr_list)
        val_list.extend(vals)

        d_list.extend(degeneracies)
        if s_label is not None:
            s_list.extend(s_label)
        if l_label is not None:
            l_list.extend(l_label)
        if j_label is not None:
            j_list.extend(j_label)
        if chi_label is not None:
            chi_list.extend(chi_label)
        n_list.extend([ival for i in enumerate(vals)])

    # Convert from <S^2> to <S>
    if eval_s2 is not None:
        eval_s2 = np.sqrt(eval_s2 + 0.25) - 0.5
    if eval_l2 is not None:
        eval_l2 = np.sqrt(eval_l2 + 0.25) - 0.5
    if eval_j2 is not None:
        eval_j2 = np.sqrt(eval_j2 + 0.25) - 0.5

    val_list, n_list, s_list, l_list, j_list, chi_list, d_list = \
            get_ordered_labels(val_list, n_list, s_list, l_list, j_list, \
            chi_list, d_list)

    return val_list, n_list, s_list, l_list, j_list, chi_list, d_list, \
            eval_s2, eval_l2, eval_j2, ent_entropy




from __future__ import print_function

import numpy as np
import os, sys, shutil, subprocess, glob, h5py, pickle, time
from tabulate import tabulate


def open_h_log(control):
    control['h_log'] = open('./cmd.log', 'w', 0)
    control['h_log'].write(
            "\n*********************************\n" +
            "             ComRISB\n" +
            "*********************************\n\n")


def read_comrisb_ini():
    vglobl = {}
    vlocal = {}
    execfile('comdmft.ini', vglobl, vlocal)
    control = vlocal['control']
    wan_hmat = vlocal['wan_hmat']
    imp = vlocal['imp']

    control['name'] = 'control'
    wan_hmat['name'] = 'wan_hmat'
    imp['name'] = 'imp'

    # timing
    control['t_rspflapw'] = 0.
    control['t_comwann'] = 0.
    control['t_interface'] = 0.
    control['t_cygutz'] = 0.
    control['t_total'] = 0.

    open_h_log(control)

    if 'comsuitedir' not in control:
        control['comsuitedir'] = os.environ.get('COMSUITE_BIN')
    if control['comsuitedir'] is None:
        raise Exception("please set comsuitedir either in environ var or "+\
                "comdmft.ini!")

    control['conv_table'] = []

    # in control
    control['top_dir'] = os.path.abspath('./')
    control["spin_orbit"] = control.get("spin_orbit", False)
    check_key_in_string('impurity_problem', control)
    check_key_in_string('impurity_problem_equivalence', control)

    # g-risb specific
    control['crystal_field'] = control.get('crystal_field', True)
    control['full_orbital_polarization'] = \
            control.get('full_orbital_polarization', False)
    control['iembed_diag'] = control.get('iembed_diag', -3)
    control['lnewton'] = control.get('lnewton', 0)
    control['spin_polarization'] = control.get('spin_polarization', False)
    control['unit'] = control.get('unit', 'ev')
    # electron density convergence criteria
    control['cc'] = control.get('cc', 1.e-4)
    # total energy convergence criteria
    control['ec'] = control.get('ec', 1.e-5)
    control['ismear'] = control.get('ismear', -1)
    # default 300K to eV
    control['delta'] = control.get('delta', 300./11604.52500617)

    control['max_iter_num_impurity'] = control.get('max_iter_num_impurity', 1)
    control['max_iter_num_outer'] = control.get('max_iter_num_outer', 20)
    control["initial_dft_dir"] = control.get('initial_dft_dir', "../dft/")
    control['initial_dft_dir'] = os.path.abspath(control['initial_dft_dir'])
    control['allfile'] = find_allfile(control['initial_dft_dir'])
    control['h_log'].write('allfile = {}\n'.format(control['allfile']))
    control["mpi_prefix"] = control.get('mpi_prefix', "")
    control["mpi_prefix_wannier"] = control.get('mpi_prefix_wannier', \
            control["mpi_prefix"])
    control["mpi_prefix_lattice"] = control.get('mpi_prefix_lattice', \
            control["mpi_prefix"])

    if 'dc_mat_to_read' in control:
        control['dc_mat_to_read'] = os.path.abspath(control['dc_mat_to_read'])

    if 'additional_unitary_transform_impurity' not in control:
        control['additional_unitary_transform_impurity'] = 0

    if control['additional_unitary_transform_impurity'] != 0:
        check_key_in_string('metal_threshold', control)
        control['h_log'].write(
                'impurity_orb_equivalence overwritten to be independent.\n')

    control['restart'] = control.get('restart', False)
    if "-c" in sys.argv:
        control['restart'] = True

    control['start_prog'] = control.get('start_prog', "initcopy")
    if "-s" in sys.argv:
        control['start_prog'] = sys.argv[sys.argv.index("-s")+1]
    control['end_prog'] = control.get('end_prog', "endconv")
    if "-e" in sys.argv:
        control['end_prog'] = sys.argv[sys.argv.index("-e")+1]

    if 'lattice_directory' not in control:
        control['lattice_directory'] = './lattice'
    if 'wannier_directory' not in control:
        control['wannier_directory'] = './wannier'
    control['lattice_directory'] = os.path.abspath(
            control['lattice_directory'])
    control['wannier_directory'] = os.path.abspath(
            control['wannier_directory'])

    if 'dc_directory' not in control:
        control['dc_directory'] = './dc'
    control['dc_directory'] = os.path.abspath(control['dc_directory'])
    if 'impurity_directory' not in control:
        control['impurity_directory'] = './impurity'
    control['impurity_directory'] = os.path.abspath(
            control['impurity_directory'])
    if 'lowh_directory' not in control:
        control['lowh_directory'] = './lowh'
    control['lowh_directory'] = os.path.abspath(control['lowh_directory'])
    if control['additional_unitary_transform_impurity'] != 0:
        check_key_in_string('initial_trans_basis', control)

    # in wan_hmat
    wan_hmat["kgrid"] = wan_hmat.get("kgrid", None)
    check_key_in_string('froz_win_max', wan_hmat)
    wan_hmat['dis_win_max'] = wan_hmat.get('dis_win_max',
            wan_hmat['froz_win_max']+40.0)

    control['proj_win_max'] = control.get('proj_win_max',
            wan_hmat['froz_win_max'])

    wan_hmat['num_iter'] = wan_hmat.get('num_iter', 0)
    wan_hmat['dis_num_iter'] = wan_hmat.get('dis_num_iter', 100)

    wan_hmat['cut_low']=wan_hmat.get('cut_low', 0.4)
    wan_hmat['cut_froz']=wan_hmat.get('cut_froz', 0.15)
    wan_hmat['cut_total']=wan_hmat.get('cut_total', 0.0)

    control['h_log'].write('top_dir {}\n'.format(control['top_dir']))
    control['h_log'].write('lattice_directory {}\n'.
            format(control['lattice_directory']))
    control['h_log'].write('wannier_directory {}\n'.
            format(control['wannier_directory']))
    control['h_log'].write('dc_directory {}\n'.format(\
            control['dc_directory']))
    control['h_log'].write('impurity_directory {}\n'.
            format(control['impurity_directory']))
    control['h_log'].write('lowh_directory {}\n'
            .format(control['lowh_directory']))

    return control, wan_hmat, imp


def find_impurity_wan(control, wan_hmat):
    num_wann = np.shape(wan_hmat['basis'])[0]
    control['impurity_wan'] = []
    for ip in range(np.shape(control['impurity_problem'])[0]):
        if control['spin_orbit']:
            if control['impurity_problem'][ip][1].lower() == 'f':
                control['impurity_wan'].append([0]*14)
                for iwan in range(num_wann):
                    if wan_hmat['basis'][iwan]['atom'] == \
                            control['impurity_problem'][ip][0] \
                            and wan_hmat['basis'][iwan]['l'] == 3:
                        if int(wan_hmat['basis'][iwan]['i']*2) == -1:
                            if int(wan_hmat['basis'][iwan]['m']*2) == -5:
                                control['impurity_wan'][ip][0] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == -3:
                                control['impurity_wan'][ip][1] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == -1:
                                control['impurity_wan'][ip][2] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == 1:
                                control['impurity_wan'][ip][3] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == 3:
                                control['impurity_wan'][ip][4] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == 5:
                                control['impurity_wan'][ip][5] = \
                                        wan_hmat['basis'][iwan]['ind']
                        elif int(wan_hmat['basis'][iwan]['i']*2) == 1:
                            if int(wan_hmat['basis'][iwan]['m']*2) == -7:
                                control['impurity_wan'][ip][6] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == -5:
                                control['impurity_wan'][ip][7] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == -3:
                                control['impurity_wan'][ip][8] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == -1:
                                control['impurity_wan'][ip][9] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == 1:
                                control['impurity_wan'][ip][10] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == 3:
                                control['impurity_wan'][ip][11] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == 5:
                                control['impurity_wan'][ip][12] = \
                                        wan_hmat['basis'][iwan]['ind']
                            elif int(wan_hmat['basis'][iwan]['m']*2) == 7:
                                control['impurity_wan'][ip][13] = \
                                        wan_hmat['basis'][iwan]['ind']
            if (control['impurity_wan'][ip].count(0) != 0):
                raise ValueError('Something wrong in find_impurity_wan.')
        else:
            if control['impurity_problem'][ip][1].lower() == 'd':
                control['impurity_wan'].append([0]*5)
                for iwan in range(num_wann):
                    if wan_hmat['basis'][iwan]['atom'] == \
                            control['impurity_problem'][ip][0]\
                            and wan_hmat['basis'][iwan]['l'] == 2:
                        if wan_hmat['basis'][iwan]['m'] == -2:
                            control['impurity_wan'][ip][0] = \
                                    wan_hmat['basis'][iwan]['ind']
                        elif wan_hmat['basis'][iwan]['m'] == -1:
                            control['impurity_wan'][ip][1] = \
                                    wan_hmat['basis'][iwan]['ind']
                        elif wan_hmat['basis'][iwan]['m'] == 0:
                            control['impurity_wan'][ip][2] = \
                                    wan_hmat['basis'][iwan]['ind']
                        elif wan_hmat['basis'][iwan]['m'] == 1:
                            control['impurity_wan'][ip][3] = \
                                    wan_hmat['basis'][iwan]['ind']
                        elif wan_hmat['basis'][iwan]['m'] == 2:
                            control['impurity_wan'][ip][4] = \
                                    wan_hmat['basis'][iwan]['ind']
            if (control['impurity_wan'][ip].count(0) != 0):
                raise ValueError(
                        'Something wrong in find_impurity_wan.')


def initial_file_directory_setup(control):
    tempdir = control['wannier_directory']
    if not os.path.isdir(tempdir):
        os.mkdir(tempdir)
    tempdir=control['lattice_directory']
    if not os.path.isdir(tempdir):
        os.mkdir(tempdir)
    tempdir=control['lowh_directory']
    if not os.path.isdir(tempdir):
        os.mkdir(tempdir)


def initial_lattice_directory_setup(control):
    os.chdir(control['lattice_directory'])
    files = glob.iglob(control['initial_dft_dir']+"/*.rst")
    for filename in files:
        shutil.copy(filename, './')
    files = glob.iglob(control['initial_dft_dir']+"/*el_density")
    for filename in files:
        shutil.copy(filename, './')
    if os.path.exists(control['initial_dft_dir']+'/kpath'):
        shutil.copy(control['initial_dft_dir']+'/kpath', './')
    if os.path.exists(control['initial_dft_dir']+'/wannier_win.dat'):
        shutil.copy(control['initial_dft_dir']+'/wannier_win.dat', './')
    else:
        raise ValueError("wannier_win.dat file not exists!")
    if os.path.exists(control['initial_dft_dir']+'/ini'):
        shutil.copy(control['initial_dft_dir']+'/ini', './')
    else:
        raise ValueError("ini file not exists!")

    iter_string = '_'+str(control['iter_num_outer'])
    shutil.copy(control['initial_dft_dir']+'/'+control['allfile']+'.out',
            control['allfile']+iter_string+'.out')

    control['h_log'].write("initial dft directory setup done.\n")
    os.chdir(control['top_dir'])


def create_comwann_ini(control, wan_hmat):
    line = open(control['lattice_directory']+"/wannier_win.dat","rb").\
            readline()
    # prefixed win_min
    wan_hmat['dis_win_min'] = wan_hmat['froz_win_min'] = float(line.split()[1])
    with open('comwann.ini', 'w') as f:
        f.write(control['lattice_directory']+'\n')
        f.write('dft\n')
        f.write(str(wan_hmat['dis_win_max'])+'\n')
        f.write(str(wan_hmat['dis_win_min'])+'\n')
        f.write(str(wan_hmat['froz_win_max'])+'\n')
        f.write(str(wan_hmat['froz_win_min'])+'\n')
        f.write(str(wan_hmat['num_iter'])+'\n')
        f.write(str(wan_hmat['dis_num_iter'])+'\n')
        f.write('0\n')
        f.write(str(wan_hmat['cut_low'])+'\n')
        f.write(str(wan_hmat['cut_froz'])+'\n')
        f.write(str(wan_hmat['cut_total'])+'\n')


def read_wan_hmat_basis(control):
    # in the wannier directory
    inip = np.loadtxt(control['wannier_directory']+'/wannier.inip')
    basis_info = []

    if (control['spin_orbit']):
        for ii in range(np.shape(inip)[0]):
            basis_info.append({'atom': int(inip[ii, 0]), \
                    'l': int(inip[ii, 1]), 'i': inip[ii, 2], \
                    'm': inip[ii, 3], 'xaxis': inip[ii, 4:7], \
                    'zaxis': inip[ii, 7:10], 'ind': ii})
    else:
        for ii in range(np.shape(inip)[0]):
            basis_info.append({'atom': int(inip[ii, 0]), \
                    'l': int(inip[ii, 1]), 'm': int(inip[ii, 2]), \
                    'xaxis': inip[ii, 3:6], 'zaxis': inip[ii, 6:9], \
                    'ind': ii})
    control['h_log'].write(\
            'reading wannier.inip to get basis information.\n')
    return basis_info


def check_key_in_string(key, dictionary):
    if key not in dictionary:
        raise ValueError('missing \''+key+'\' in '+dictionary['name'])


def labeling_file(filename, iter_string):
    dirname = os.path.abspath(os.path.dirname(filename))
    filenameonly = os.path.basename(filename)
    temp = filenameonly.split('.')
    shutil.copy(dirname+'/'+filenameonly, dirname+"/" +
                '.'.join(temp[0:-1])+iter_string+'.'+temp[-1])


def write_conv(control):
    os.chdir(control['lattice_directory'])
    iter_string = '_' + str(control['iter_num_outer'])
    with open('./dft'+iter_string+'.out', "r") as f:
        for line in f:
            if "charge density" in line:
                line = line.split()
                delta_rho=float(line[3])
            elif "ETOT" in line:
                line = line.split()
                etot = float(line[4])
        control['conv_table'].append([control['iter_num_outer'], \
                delta_rho, etot])
    with h5py.File(control['lowh_directory']+"/GLOG.h5", "r") as f:
        mu = f["/e_fermi"][0]
        err_risb = f["/rl_maxerr"][0]
        control['conv_table'][-1].extend([mu, err_risb])
        rmat2 = f["/BND_R"][()].swapaxes(1,2)
    # check quasiparticle weight
    zmin = 100.
    for rmat in rmat2:
        zmat = rmat.T.conj().dot(rmat)
        w = np.linalg.eigvalsh(zmat)
        zmin = min(zmin, np.amin(w))
    control['conv_table'][-1].append(zmin)
    os.chdir(control['top_dir'])
    with open("convergence.log", "wb") as f:
        f.write(tabulate(control['conv_table'], \
                headers=['i_outer', 'delta_rho', 'etot', "mu", \
                'err_risb', "min_z"], \
                numalign="right",  floatfmt=".8f"))


def check_wannier_function_input(control, wan_hmat):
    os.chdir(control['wannier_directory'])
    create_comwann_ini(control, wan_hmat)
    if os.path.exists(control['top_dir']+'/local_axis.dat'):
        shutil.copy(control['top_dir']+'/local_axis.dat', './')
    os.chdir(control['top_dir'])


def run_dft(control):
    os.chdir(control['lattice_directory'])
    iter_string = '_'+str(control['iter_num_outer'])
    cmd = control['mpi_prefix_lattice'] + ' ' + \
            control['comsuitedir'] + "/rspflapw.exe"
    hlog_time(control['h_log'], "rspflapw start")
    t_start = time.time()
    with open(control['lattice_directory']+'/dft.out', 'w') as logfile:
        ret = subprocess.call(cmd, shell=True, stdout=logfile, \
                stderr=logfile)
        if ret != 0:
            raise ValueError("Error in dft. Check dft.out for error message.")
    hlog_time(control['h_log'], "rspflapw end")
    t_end = time.time()
    control['t_rspflapw'] += t_end-t_start
    allfile=control['allfile']
    labeling_file('./'+allfile+'.out',iter_string)
    shutil.move('./dft.out', './dft'+iter_string+'.out')
    control['h_log'].write("dft calculation done.\n")
    os.chdir(control['top_dir'])


def prepare_dft_input(control):
    shutil.copy(control['lowh_directory']+"/wannier_den_matrix.dat",
            control['lattice_directory'])
    control['h_log'].write("prepare_dft_input done.\n")


def hlog_time(f, prestr, endl=""):
    f.write("{} at {} {}".format(prestr, time.strftime("%d/%m/%Y %H:%M:%S"), \
            endl))


def wannier_run(control,wan_hmat,fullrun=True):
    os.chdir(control['wannier_directory'])
    if fullrun:
        cmd = control['mpi_prefix_wannier']+' '+\
                control['comsuitedir']+"/ComWann"
        control['h_log'].write(cmd+"\n")

        hlog_time(control['h_log'], "comwann start")
        t_start = time.time()
        with open(control['wannier_directory']+'/comwann.out', 'w') \
                as logfile:
            ret = subprocess.call(cmd, shell=True, stdout = logfile, \
                    stderr = logfile)
            if ret != 0:
                raise ValueError(
                        "Error in comwann. Check comwann.out or OUT.")
        hlog_time(control['h_log'], "end", endl="\n")
        t_end = time.time()
        control['t_comwann'] += t_end-t_start

        iter_string = '_' + str(control['iter_num_outer'])
        shutil.move('./wannier.wout', './wannier'+iter_string+'.wout')

    wan_hmat['basis'] = read_wan_hmat_basis(control)
    find_impurity_wan(control, wan_hmat)
    control['h_log'].write("control['impurity_wan']: {}\n".format(\
            control['impurity_wan']))
    os.chdir(control['top_dir'])


def bool2yesno(boolean):
    if boolean:
        return "y"
    else:
        return "n"

def get_locrot_list(wan_hmat):
    iatm = -1
    lrot_list = []
    for basis in wan_hmat["basis"]:
        if basis["atom"] != iatm:
            x = np.asarray([float(e) for e in basis["xaxis"]])
            z = np.asarray([float(e) for e in basis["zaxis"]])
            y = np.cross(z, x)
            lrot_list.append(np.asarray([x,y,z]).T)
            iatm = basis["atom"]
    return lrot_list


def init_grisb(control, imp):
    with h5py.File("ginit.h5", "r") as f:
        symbols = f["/struct/symbols"][()]

    unique_df_list = []
    unique_f_list_ev = []
    unique_j_list_ev = []
    unique_u_list_ev = []
    unique_nf_list = []
    corr_atm_list = []
    corr_df_list = []
    for i,impurity in enumerate(control['impurity_problem']):
        corr_atm_list.append(impurity[0]-1) # to zero-base
        corr_df_list.append(impurity[1])
        if len(corr_atm_list) == 1 or \
                symbols[corr_atm_list[-1]] != symbols[corr_atm_list[-2]]:
            unique_df_list.append(impurity[1])
            f_list = np.zeros(4)
            for ifs,fs in enumerate(["f0", "f2", "f4", "f6"]):
                if fs in imp[str(i+1)]:
                    f_list[ifs] = imp[str(i+1)][fs]
                elif fs.upper() in imp[str(i+1)]:
                    f_list[ifs] = imp[str(i+1)][fs.upper()]
            unique_f_list_ev.append(f_list)
            unique_u_list_ev.append(f_list[0])
            if "nominal_n" in imp[str(i+1)]:
                nf = imp[str(i+1)]["nominal_n"]
                unique_nf_list.append([nf/2., nf/2.])
            if corr_df_list[-1].lower() == "d":
                j_hund = f_list[1]*(1.0 + 0.625)/14.0
            elif corr_df_list[-1].lower() == "f":
                j_hund = f_list[1]*(286.0 + 195.0*0.668 + \
                        250.0 *0.494)/6435.0
            elif corr_df_list[-1].lower() == "p":
                j_hund = f_list[1]/5.
            elif corr_df_list[-1].lower() == "s":
                j_hund = 0.
            else:
                raise ValueError("Not implemented for l = {}!".format(\
                        corr_df_list[-1]))
            unique_j_list_ev.append(j_hund)

        if len(corr_atm_list) > 1:
            # ensure atoms in ascendig order.
            assert corr_atm_list[-1] >= corr_atm_list[-2], \
                    "please specify impurities in asceding order!"

    if len(unique_nf_list) == 0:
        ldc = 12
    else:
        ldc = 2
    # Sanity check
    for i in range(1,len(corr_df_list)):
        if control["impurity_problem_equivalence"][i] == \
                control["impurity_problem_equivalence"][i-1]:
            assert corr_df_list[i] == corr_df_list[i-1], \
                    "correlated orbitals incompatible with equiv_list!"
    # setup unique_corr_symbol_list
    unique_corr_symbol_list = []
    for i in corr_atm_list:
        if symbols[i] not in unique_corr_symbol_list:
            unique_corr_symbol_list.append(symbols[i])
    idx_equivalent_atoms = []

    for i in range(len(symbols)):
        if i == 0:
            idx_equivalent_atoms.append(0)
            continue
        elif i in corr_atm_list:
            idx = corr_atm_list.index(i)
            if idx > 0:
                if control["impurity_problem_equivalence"][idx] \
                        == control["impurity_problem_equivalence"][idx-1]:
                    idx_equivalent_atoms.append(idx_equivalent_atoms[-1])
                    continue
        idx_equivalent_atoms.append(idx_equivalent_atoms[-1]+1)

    with h5py.File("ginit.h5", "a") as f:
        f["/usrqa/crystal_field"] = bool2yesno(\
                control["crystal_field"])
        f["/usrqa/full_orbital_polarization"] = bool2yesno(\
                control["full_orbital_polarization"])
        f["/usrqa/idx_equivalent_atoms"] = idx_equivalent_atoms
        f["/usrqa/iembeddiag"] = control["iembed_diag"]
        f["/usrqa/ldc"] = ldc
        f["/usrqa/lnewton"] = control['lnewton']
        f["/usrqa/spin_orbit_coup"] = bool2yesno(\
                control['spin_orbit'])
        f["/usrqa/spin_polarization"] = bool2yesno(\
                control['spin_polarization'])
        f["/usrqa/u_matrix_type"] = 3
        f["/usrqa/unique_corr_symbol_list"] = unique_corr_symbol_list
        f["/usrqa/unique_df_list"] = unique_df_list
        f["/usrqa/unique_j_list_ev"] = unique_j_list_ev
        f["/usrqa/unique_u_list_ev"] = unique_u_list_ev
        f["/usrqa/unique_f_list_ev"] = unique_f_list_ev
        if len(unique_nf_list) > 0:
            f["/usrqa/unique_nf_list"] = unique_nf_list
        f["/usrqa/unit"] =  control['unit']
    from pygrisb.gutz.init import initialize as ginit
    ginit()


def gwannier_run(control, wan_hmat, imp, icycle):
    os.chdir(control['lowh_directory'])
    lrot_list = get_locrot_list(wan_hmat)
    params = {}
    params["corbs_list"] = control['impurity_wan']
    params["wpath"] = control['wannier_directory']
    params["lrot_list"] = lrot_list
    params["icycle"] = icycle
    params["ismear"] = control['ismear']
    params["delta"] = control['delta']
    if control['spin_orbit']:
        params["iso"] = 2
    params["lprefix"] = control['allfile']

    # prepare parameter file for mpirun
    with open("gwannier_params.pkl", "wb") as f:
        pickle.dump(params, f, protocol=pickle.HIGHEST_PROTOCOL)

    cmd = control['mpi_prefix_wannier'] + ' ' + \
            control['comsuitedir'] + "/gwannier.py"
    control['h_log'].write(cmd+"\n")
    hlog_time(control['h_log'], "gwannier start")
    t_start = time.time()
    with open(control['lowh_directory']+'/gwannier.out', 'w') as f:
        ret = subprocess.call(cmd, shell=True, stdout = f, \
                stderr = f)
    hlog_time(control['h_log'], "end", endl="\n")
    t_end = time.time()
    control['t_interface'] += t_end-t_start
    assert ret == 0, "Error in gwannier. Check gwannier.out."

    if icycle <= 1:
        init_grisb(control, imp)

    cmd = control['mpi_prefix_wannier'] + ' ' + \
            control['comsuitedir'] + "/CyGutz -r 0 " + \
            "-p " + control['comsuitedir']
    control['h_log'].write(cmd+"\n")
    hlog_time(control['h_log'], "cygutz start")
    t_start = time.time()
    with open(control['lowh_directory']+'/grisb.out', 'w') as f:
        ret = subprocess.call(cmd, shell=True, stdout = f, \
                stderr = f)
    hlog_time(control['h_log'], "end", endl="\n")
    t_end = time.time()
    control['t_cygutz'] += t_end-t_start
    assert ret == 0, "Error in grisb. Check grisb.out."
    shutil.copy("./WH_RL_OUT.h5", "./WH_RL_INP.h5")
    shutil.copy("./GUTZ.LOG", "SAVE_GUTZ.LOG")

    cmd = control['mpi_prefix_wannier'] + ' ' + \
            control['comsuitedir'] + "/gwannden.py"
    control['h_log'].write(cmd+"\n")
    hlog_time(control['h_log'], "gwannden start")
    t_start = time.time()
    with open(control['lowh_directory']+'/gwannden.out', 'w') as f:
        ret = subprocess.call(cmd, shell=True, stdout = f, \
                stderr = f)
    hlog_time(control['h_log'], "end", endl="\n")
    t_end = time.time()
    control['t_interface'] += t_end-t_start
    assert ret == 0, "Error in gwannden. Check gwannden.out."
    os.chdir(control['top_dir'])


def find_allfile(dft_dir):
    files = glob.iglob(dft_dir+"/*.rst")
    for filename in files:
        temp = filename[:-4].split('/')[-1].split('_')
        if len(temp)==1 and temp[0]!='info':
            allfile = temp[0]
    try:
        return allfile
    except:
        raise ValueError("failed to locate *.rst files in {}!".format(\
                dft_dir))


def dft_risb(control, wan_hmat, imp):
    control['h_log'].write("\n\n")
    control['iter_num_outer'] = 1
    while control['iter_num_outer'] <= control['max_iter_num_outer']:
        control['h_log'].write(\
                "************************************************\n"+ \
                "iteration: {}\n".format(control['iter_num_outer'])+ \
                "************************************************\n")
        if control['iter_num_outer'] == 1 and \
                control['start_prog'] in ["initcopy"]:
            if not control['restart']:
                initial_lattice_directory_setup(control)

        fullrun = control['iter_num_outer'] > 1 or \
                control['start_prog'] in ["initcopy", "wannier"]
        check_wannier_function_input(control, wan_hmat)
        wannier_run(control, wan_hmat, fullrun)
        if control['iter_num_outer'] > 1 or \
                control['start_prog'] in ["initcopy", "wannier", "gwann"]:
            gwannier_run(control, wan_hmat, imp, control['iter_num_outer'])
        if control['iter_num_outer'] > 1 or \
                control['start_prog'] in ["initcopy", "wannier", \
                "gwann", "dft"]:
            prepare_dft_input(control)
            run_dft(control)
        if control['end_prog'] in ["dft"]:
            break
        write_conv(control)
        # convergence

        if control['iter_num_outer'] > 1 and \
                control['conv_table'][-1][1] < control['cc'] and \
                abs(control['conv_table'][-1][2]- \
                control['conv_table'][-2][2]) < control['ec']:
            break
        control['iter_num_outer']=control['iter_num_outer']+1


def log_time(control):
    control['h_log'].write("\ntimings:\n")
    control['h_log'].write("  rspflapw : {:.1f}s\n".format(\
            control['t_rspflapw']))
    control['h_log'].write("  comwann  : {:.1f}s\n".format(\
            control['t_comwann']))
    control['h_log'].write("  interface: {:.1f}s\n".format(\
            control['t_interface']))
    control['h_log'].write("  cygutz   : {:.1f}s\n".format(\
            control['t_cygutz']))
    control['h_log'].write("  subtotal : {:.1f}s\n".format(\
            control['t_rspflapw']+control['t_comwann']+\
            control['t_interface']+control['t_cygutz']))
    control['h_log'].write("  total    : {:.1f}s\n".format(\
            control['t_total']))



if __name__ == '__main__':
    t_start = time.time()
    control,wan_hmat,imp = read_comrisb_ini()
    initial_file_directory_setup(control)
    dft_risb(control, wan_hmat, imp)
    t_end = time.time()
    control['t_total'] = t_end-t_start
    log_time(control)
    control['h_log'].close()



from __future__ import print_function
# analyze the embeding hamiltonian
import os, h5py, numpy, shutil
from scipy.linalg import block_diag
from pygrisb.math.matrix_util import \
        get_utrans_orbfast_supdn_to_spin_fast_supdn
from pygrisb.math.matrix_util import unitary_transform_coulomb_matrix


def twrite_2d_array_real(a, fname, shreshold=1.e-10):
    '''write 2d array (real) in text form of i, j, a_ij.
    '''
    with open(fname, 'w') as f:
        for i, a1 in enumerate(a):
            for j, a12 in enumerate(a1):
                if numpy.abs(a12) > shreshold:
                    f.write('{:2d} {:2d} {:16.12f}\n'.format(\
                            i+1, j+1, a12.real))


def twrite_2d_array_cmplx(a, fname, shreshold=1.e-10):
    '''write 2d array (real) in text form of i, j, a_ij.
    '''
    with open(fname, 'w') as f:
        for i, a1 in enumerate(a):
            for j, a12 in enumerate(a1):
                if numpy.abs(a12) > shreshold:
                    f.write('{:2d} {:2d} {:16.12f} {:16.12f}\n'.format(\
                            i+1, j+1, a12.real, a12.imag))


def twrite_u_matrix_real(v2e, fname, shreshold=1.e-10):
    '''write u-matrix in text form of i, k, l, j, v2e_(ij)(kl).
    '''
    with open(fname, 'w') as f:
        for i, v1 in enumerate(v2e):
            for j, v12 in enumerate(v1):
                for k, v123 in enumerate(v12):
                    if i == k:
                        continue
                    for l, v1234 in enumerate(v123):
                        if j == l:
                            continue
                        if numpy.abs(v1234) > shreshold:
                            # order of cd_i cd_k c_l c_j
                            # absorbing 1/2 factor
                            f.write('{:2d} {:2d} {:2d} {:2d} {:16.12f}\n'\
                                    .format(i+1, k+1, l+1, j+1, \
                                    v1234.real/2))


def twrite_u_matrix_cmplx(v2e, fname, shreshold=1.e-10):
    '''write u-matrix in text form of i, k, l, j, v2e_(ij)(kl).
    '''
    with open(fname, 'w') as f:
        for i, v1 in enumerate(v2e):
            for j, v12 in enumerate(v1):
                for k, v123 in enumerate(v12):
                    if i == k:
                        continue
                    for l, v1234 in enumerate(v123):
                        if j == l:
                            continue
                        if numpy.abs(v1234) > shreshold:
                            # order of cd_i cd_k c_l c_j
                            # absorbing 1/2 factor
                            f.write(('{:2d} {:2d} {:2d} {:2d}'+
                                    ' {:16.12f} {:16.12f}\n')\
                                    .format(i+1, k+1, l+1, j+1, \
                                    v1234.real/2, v1234.imag/2))


def get_u_sab2cshupdn(imp=1):
    '''get unitary transformation from symmetry-adapted basis
    to complex spherical harmonics basis with
    faster spin index ordered as up and down.
    '''
    # default basis to symmetry adapted basis transformation
    with h5py.File('GPARAM.h5', 'r') as f:
        u_db2sab = f['/IMPURITY_{}/DB_TO_SAB'.format(imp)][()].T

    u_sf2of = get_utrans_orbfast_supdn_to_spin_fast_supdn( \
            u_db2sab.shape[0]).T
    u_sf2sab = u_sf2of.dot(u_db2sab)
    u_sab2sf = u_sf2sab.T.conj()
    return u_sab2sf


def h5gen_embedh_spin_updn(imp=1, lwrt_csh=False, lv2e=False):
    # embedding hamiltonian parameters
    # upper case path indicates fortran convention.
    with h5py.File('EMBED_HAMIL_{}.h5'.format(imp), 'r') as f:
        daalpha = f['/D'][()].T
        lambdac = f['/LAMBDA'][()].T
        h1e     = f['/H1E'][()].T
        if lv2e:
            v2e = f['/V2E'][()].T
        else:
            v2e = None

    u_sab2sf = get_u_sab2cshupdn(imp=imp)
    # basis transformation
    daalpha_ = u_sab2sf.T.dot(daalpha).dot(u_sab2sf.conj())
    lambdac_ = u_sab2sf.T.conj().dot(lambdac).dot(u_sab2sf)
    h1e_ = u_sab2sf.T.conj().dot(h1e).dot(u_sab2sf)

    if v2e is not None:
        v2e_ = unitary_transform_coulomb_matrix(v2e, u_sab2sf)
        # double check
        # v2e_ = numpy.einsum('ijkl,pi,jq,rk,ls->pqrs', v2e, \
        #         u_sab2sf.T.conj(), u_sab2sf, \
        #         u_sab2sf.T.conj(), u_sab2sf)
    else:
        v2e_= None

    if lwrt_csh:
        shutil.copy('EMBED_HAMIL_{}.h5'.format(imp), \
                'EMBED_HAMIL_CSH_{}.h5'.format(imp))
        with h5py.File('EMBED_HAMIL_CSH_{}.h5'.format(imp), 'a') as f:
            f['/D'][()] = daalpha_.T
            f['/LAMBDA'][()] = lambdac_.T
            f['/H1E'][()] = h1e_.T
            if v2e_ is not None:
                f['/V2E'][()] = v2e_.T
    return h1e_, lambdac_, daalpha_, v2e_


def h5wrt_rembed_hamil(h1e, lambdac, daalpha, v2e, imp=1):
    '''write the real version of the embedding Hamiltonian in hdf5 format.
    '''
    h1e, lambdac, daalpha, v2e = [get_real_with_chk(a) for a in [h1e, \
            lambdac, daalpha, v2e]]
    fdst = "EMBED_HAMIL_{}r.h5".format(imp)
    if os.path.isfile(fdst):
        with h5py.File(fdst, "a") as f:
            f['/D'][()] = daalpha.T
            f['/LAMBDA'][()] = lambdac.T
            f['/H1E'][()] = h1e.T
    else:
        shutil.copy('EMBED_HAMIL_{}.h5'.format(imp), fdst)
        with h5py.File(fdst, "a") as f:
            del f['/D'], f['/LAMBDA'], f['/H1E'], f['/V2E']
            f['/D'] = daalpha.T
            f['/LAMBDA'] = lambdac.T
            f['/H1E'] = h1e.T
            f['/V2E'] = v2e.T


def get_whole_h1e(h1e, lambdac, daalpha):
    '''get the total one-body part of the embedding hamiltonian
    given the components of impurity, bath and hybridization.
    '''
    norb = h1e.shape[0]
    v1e = block_diag(h1e, -lambdac)
    v1e[:norb, norb:] = daalpha.T
    v1e[norb:, :norb] = daalpha.conj()
    return v1e


def wrt_text_rembed(h1e, lambdac, daalpha, v2e, imp=1, shreshold=1.e-10):
    '''write the real embedding hamiltonian parameters in text format.
    '''
    v1e = get_whole_h1e(h1e, lambdac, daalpha)

    # taking care of f_b fd_a = fd_a f_b + \delta_a,b.
    # v1e += numpy.eye(v1e.shape[0])*lambdac.trace()/lambdac.shape[0]
    # but here the convention is for a typical molecule.
    v1e, v2e = [get_real_with_chk(a) for a in [v1e, v2e]]

    # write one-body part
    twrite_2d_array_real(v1e, 'H1E_{}.INP'.format(imp), shreshold=shreshold)

    # write two-body part
    if v2e is not None:
        twrite_u_matrix_real(v2e, 'V2E_{}.INP'.format(imp),
                shreshold=shreshold)


def wrt_text_cembed(h1e, lambdac, daalpha, v2e, imp=1):
    '''write the complex embedding hamiltonian parameters in text format.
    '''
    v1e = get_whole_h1e(h1e, lambdac, daalpha)

    # taking care of f_b fd_a = fd_a f_b + \delta_a,b.
    # v1e += numpy.eye(v1e.shape[0])*lambdac.trace()/lambdac.shape[0]
    # but here the convention is for a typical molecule.

    # write one-body part
    twrite_2d_array_cmplx(v1e, 'H1E_{}.INP'.format(imp), shreshold=1.e-10)

    # write two-body part
    if v2e is not None:
        twrite_u_matrix_cmplx(v2e, 'V2E_{}.INP'.format(imp))


def get_dm_cshupdn(imp=1):
    '''get the density matrix in complex spherical harmonics basis
    with faster spin index ordered as up, down.
    '''
    with h5py.File('EMBED_HAMIL_RES_{}.h5'.format(imp), 'r') as f:
        dm = f['/DM'][()].T
    u_sab2sf = get_u_sab2cshupdn(imp=imp)
    u2_sab2sf = block_diag(u_sab2sf, u_sab2sf)
    dm = u2_sab2sf.T.dot(dm).dot(u2_sab2sf.conj())
    if numpy.max(numpy.abs(dm.imag)) < 1.e-10:
        dm = dm.real
    numpy.savetxt('dm.dat', dm)


def get_real_with_chk(a, threshold=1.e-10):
    '''return real version of a with a warning if the imaginary part
    is larger than the threshold.
    '''
    if a is None:
        return a
    max_imag = numpy.max(numpy.abs(a.imag))
    if max_imag > 1.e-10:
        print(' maximal imaginary part = {}'.format(max_imag))
    return a.real


def get_res_idmrg(imp=1):
    '''get density matrix and energy from idmrg calculation.
    '''
    # read in density matrix (real)
    fname = "GDMRG_{}.OUT".format(imp)
    dm = numpy.loadtxt(fname)
    if dm.shape[1] == dm.shape[0]*2:
        dm = dm.view(complex)

    # get transformation matrix to symmetry adapted basis
    line = open(fname, "r").readline()
    e_mol = float(line.split()[1])

    return dm, e_mol


def get_res_rspci_mott_onfly(imp=1):
    '''get density matrix and energy from rspci_mott_onfly calculation.
    '''
    with h5py.File("EMBED_HAMIL_RES_{}r.h5".format(imp), "r") as f:
        dm = f["/DM"][()].T
        e_mol = f["/emol"][0]
    return dm, e_mol


def h5wrt_dm_sab_cmplx(dm, e_mol, imp=1):
    '''transform the density matrix (real) into symmetry adapted basis
    and write in EMBEB_HAMIL_{IMP}.h5 file in complex format.
    '''
    # get the comp_sph_harm to sab transformation
    u_sab2sf = get_u_sab2cshupdn(imp=imp)

    # include the transformation for the bath
    u2_sab2sf = block_diag(u_sab2sf, u_sab2sf)

    # unitary transformation
    dm = u2_sab2sf.conj().dot(dm).dot(u2_sab2sf.T)
    with h5py.File("EMBED_HAMIL_RES_{}.h5".format(imp), 'w') as f:
        f["/DM"] = dm.T
        f["/emol"] = [e_mol]
        f["/dimv"] = [0]


def h5wrt_dm_sab_rc(dm, e_mol, imp=1):
    '''transform the density matrix (real) into symmetry adapted basis
    and write in EMBEB_HAMIL_{IMP}.h5 file in complex format.
    '''
    # cast to complex
    dm = numpy.array(dm, dtype=complex)
    h5wrt_dm_sab_cmplx(dm, e_mol, imp=imp)


def chk_consistent_dm(imp=1):
    '''consistent check for dm with unitary transformations.
    '''
    dm = numpy.loadtxt('dm.dat')
    with h5py.File('EMBED_HAMIL_CSH_{}.h5'.format(imp), 'r') as f:
        daalpha = f['/D'][()].T
        lambdac = f['/LAMBDA'][()].T
        h1e     = f['/H1E'][()].T
    v1e = get_whole_h1e(h1e, lambdac, daalpha)
    res1 = numpy.sum(dm*v1e)

    with h5py.File('EMBED_HAMIL_RES_{}.h5'.format(imp), 'r') as f:
        dm = f['/DM'][()].T
    with h5py.File('EMBED_HAMIL_{}.h5'.format(imp), 'r') as f:
        daalpha = f['/D'][()].T
        lambdac = f['/LAMBDA'][()].T
        h1e     = f['/H1E'][()].T
    v1e = get_whole_h1e(h1e, lambdac, daalpha)
    res2 = numpy.sum(dm*v1e)
    print(res1, res2)


def embed_hamil_to_ctext(imp=1):
    with h5py.File("EMBED_HAMIL_{}.h5".format(imp), 'r') as f:
        daalpha = f['/D'][()].T
        lambdac = f['/LAMBDA'][()].T
        h1e     = f['/H1E'][()].T
        v2e     = f['/V2E'][()].T
    wrt_text_cembed(h1e, lambdac, daalpha, v2e, imp=imp)



if __name__ == '__main__':
    _ = h5gen_embedh_spin_updn(lwrt_csh=True, lv2e=True)



import os, sys, subprocess
from pygrisb.gutz import embedh


def create_idmrg_ctl_file(imp=1, N=14, Npart=14, Nsweeps=5, ecut=1.e-6,
        maxM=200):
    '''write idmrg control file.
    '''
    fname = "GDMRG_{}.CTL".format(imp)
    if os.path.isfile(fname):
        return
    with open(fname, "w") as f:
        f.write(
'''input
    {{
    N = {}
    Npart = {}
    ConserveSz = no
    quiet = yes
    Nsweeps = {}
    Ecut = {}
    maxM = {}
    }}
'''.format(N, Npart, Nsweeps, ecut, maxM))


def driver_idmrg(imp=1):
    '''dmrg solver based on ITensor.
    '''
    if "-i" in sys.argv:
        imp = int(sys.argv[sys.argv.index("-i")+1])
    if os.path.isfile("V2E_{}.INP".format(imp)):
        lv2e = False
    else:
        lv2e = True
    h1e, lambdac, daalpha, v2e = embedh.h5gen_embedh_spin_updn(
            imp=imp, lv2e=lv2e)
    embedh.wrt_text_cembed(h1e, lambdac, daalpha, v2e)
    N = Npart = h1e.shape[0]
    create_idmrg_ctl_file(imp=imp, N=N, Npart=Npart)
    cmd = [os.environ["WIEN_GUTZ_ROOT2"]+"/exe_idmrg", str(imp)]
    with open("GDMRG_{}.LOG".format(imp), "w") as f:
        subprocess.call(cmd, stdout=f)
    dm, e_tot = embedh.get_res_idmrg(imp=imp)
    embedh.h5wrt_dm_sab_cmplx(dm, e_tot, imp=imp)


if __name__ == "__main__":
    driver_idmrg()



import os, subprocess


def get_env_dict(key="SLURM_"):
    '''get a list of environment variables containing key word of key.
    '''
    envs = {}
    for e in os.environ:
        if key in e:
            envs[e] = os.environ[e]
    return envs


def unset_environ(envs):
    '''unset the list of environment variables envs.
    '''
    for e in envs:
        if e in os.environ:
            del os.environ[e]


def set_environ(envs):
    '''set the list of environment variables envs.
    '''
    for e in envs:
        if e not in os.environ:
            os.environ[e] = envs[e]




import numpy as np
from ase.dft import kpoints
import pygrisb.gutz.ginput as ginput
import pygrisb.model.tbASE as tb

# The following is a simple test for the 1-d Hubbard model.

a = tb.AtomsTB("N", [(0, 0, 0)], cell=(1, 1, 1))
a.set_orbitals_spindeg()
aTB = tb.TB(a)
aTB.set_hop([
    ((1, 0, 0), 0, 0, -1),
    ((-1, 0, 0), 0, 0, -1),
    ((0, 0, 0), 0, 0,  0), ])

kps_size = (100, 1, 1)
kps = kpoints.monkhorst_pack(kps_size)

num_k = len(kps)
kps_wt = 1.0 / num_k * np.ones((num_k))
if aTB.Atoms.spindeg:
    kps_wt *= 2
num_e = 1.0
num_band_max = 1

# GPARAMBANDS.h5
h1e_list = [np.array([[0, 0], [0, 0]], dtype=np.complex)]
ginput.save_gparambands(kps_wt, num_e, num_band_max, h1e_list=h1e_list)

sigma_list = [np.identity(2, dtype=np.int32)]
v2e = np.zeros((2, 2, 2, 2), dtype=np.complex)
v2e[0, 0, 0, 0] = v2e[0, 0, 1, 1] = v2e[1, 1, 0, 0] = v2e[1, 1, 1, 1] = 6.0
sz = np.asarray(np.diag((1,-1)),dtype=np.complex)

# GPARAM.h5
ginput.save_gparam(sigma_list=sigma_list, iembeddiag=-1, v2e_list=[v2e],
        sz_list=[sz])

# BAREHAM_0.h5
aTB.save_bareham(kps)



from __future__ import print_function
import sys, os, numpy, h5py
from pygrisb.mbody.coulomb_matrix import U_J_to_radial_integrals, \
        radial_integrals_to_U_J


def h5save_usr_qa_setup(material, log=sys.stdout):
    '''
    A list of questions to be answered to initialize the CyGutz job.
    '''
    f = h5py.File('ginit.h5', 'a')
    if '/usrqa' in f:
        f.close()
        return
    usr_input = open("input.slog", 'w')
    print('\n' + " User inputs to initialize the G-RISB job.")
    # cut-off distance to determine the rotation group.
    dist_cut = -1.0
    if '-c' in sys.argv:
        dist_cut = float(sys.argv[sys.argv.index('-c') + 1])
        print(" The uniform dist_cut for extracting a centered" + \
                " cluster \n for symmetry evaluation = {}\n".format(dist_cut),
                file=log)
    f['/usrqa/dist_cut'] = dist_cut

    unit = 'rydberg'
    if '-u' in sys.argv:
        unit = sys.argv[sys.argv.index('-u') + 1]
    f['/usrqa/unit'] = unit

    # Spin symmetry breaking
    if '-spl' in sys.argv:
        spin_polarization = sys.argv[sys.argv.index('-spl') + 1]
    else:
        spin_polarization = get_usr_input(
                "\n Do you want to BREAK SPIN-SYMMETRY?", ['y', 'n'])
        print(spin_polarization, file=usr_input)
    f['/usrqa/spin_polarization'] = spin_polarization

    # Ferromagnetic or not
    if 'y' == spin_polarization:
        ferromagnetism = get_usr_input(
                "\n Is it a ferromagnetic (FM) calculation?", ['y', 'n'])
        print(ferromagnetism, file=usr_input)
        f['/usrqa/ferromagnetism'] = ferromagnetism

    # Orbital symmetry breaking
    if '-opl' in sys.argv:
        orbital_polarization = sys.argv[sys.argv.index('-opl') + 1]
    else:
        orbital_polarization = get_usr_input(
                "\n Do you want to COMPLETELY break orbital-symmetry?",
                ['y', 'n'])
        print(orbital_polarization, file=usr_input)
    f['/usrqa/full_orbital_polarization'] = orbital_polarization

    # Spin-orbit interaction.
    if '-soc' in sys.argv:
        spin_orbit_coup = sys.argv[sys.argv.index('-soc') + 1]
    else:
        spin_orbit_coup = get_usr_input(
                "\n Do you want to take into account the SPIN-ORBIT" +
                " interaction?", ['y', 'n'])
        print(spin_orbit_coup, file=usr_input)
    f['/usrqa/spin_orbit_coup'] = spin_orbit_coup

    if 'y' == spin_polarization == spin_orbit_coup:
        print(' Warning: magnetism with spin-orbit coupling is in ' +
                'experimental stage.')
        if 'y' == ferromagnetism:
            ferro_magmom_direction = get_direction(log=usr_input)

    # Crystal field
    if 'n' in orbital_polarization:
        if '-cf' in sys.argv:
            crystal_field = sys.argv[sys.argv.index('-cf') + 1]
        else:
            crystal_field = get_usr_input(
                    "\n Do you want to take into account the" +
                    " CRYSTAL FIELD effect?", ['y', 'n'])
            print(crystal_field, file=usr_input)
    else:
        crystal_field = 'y'
    f['/usrqa/crystal_field'] = crystal_field

    # Coulomb U
    if '-um' in sys.argv:
        lhub = sys.argv[sys.argv.index('-um') + 1]
    else:
        print("\n Please select the method to parametrize Coulomb U-matrix.\n"+
                " LHUB = 1: Slater-Condo parametrization (U,J).\n" +
                "        3: Slater-Condo parametrization (F-integral).\n" +
                "        2: Kanamori parametrization (useful for models).\n" +
                "        0: Manual input.")
        lhub = get_usr_input(" Please select LHUB: ", ['1', '2', "3", '0'])
        usr_input.write(lhub + '\n')
    lhub = int(lhub)
    f['/usrqa/u_matrix_type'] = lhub

    # Choose double counting functional
    if '-dc' in sys.argv:
         ldc = sys.argv[sys.argv.index('-dc') + 1]
    else:
        print("\n Please select method for U-interaction double counting.\n" +
                " LDC = 12: Recommended for LDA+G-RISB calculations.\n" +
                "           Fully-localized-limit (FLL) double counting. \n" +
                "           (updating Vdc at each charge iteration.) \n" +
                "        2: Fix double counting potential \n" +
                "           (keep same Vdc/n0 at each charge iteration,\n" +
                "           n0 to be provided.) \n" +
                "        1: FLL double counting potential \n" +
                "           (n0 self-consistently determined.) \n" +
                "        0:  No double counting (useful for models). ")
        ldc = get_usr_input(" Please select LDC: ", ['12', '0', '1', '2'])
        usr_input.write(ldc + '\n')
    ldc = int(ldc)
    f['/usrqa/ldc'] = ldc

    # Equivalent atom indices
    if '-eqidx' in sys.argv:
        string_idx_equivalent_atoms = sys.argv[sys.argv.index('-eqidx') + 1]
        idx_equivalent_atoms = [
                int(s) for s in string_idx_equivalent_atoms.split()]
    else:
        idx_equivalent_atoms = material.get_EquivalentAtoms()
        yn = get_usr_input("\n Symmetrically-equivalent atom indices: " \
                + ''.join("%2d " % (i) for i in idx_equivalent_atoms) +
                "\n (note: '0 0 0 1 1' means 1-3 and 4-5 are two" +
                " inequivalent atoms). \n Accept?", ['y', 'n'])
        print(yn, file=usr_input)

        if yn == 'n':
            while True:
                string_idx_equivalent_atoms = input(
                    " Enter user-defined equivalent atom indices: ")
                yn1 = get_usr_input(
                        "\n User-defined equivalent atom indices: " +
                        string_idx_equivalent_atoms + ". Accept?", ['y', 'n'])
                if yn1 == 'y':
                    idx_equivalent_atoms = [
                            int(s) for s in string_idx_equivalent_atoms.split()]
                    print(string_idx_equivalent_atoms, file=usr_input)
                    print(yn1, file=usr_input)
                    break

    f['/usrqa/idx_equivalent_atoms'] = idx_equivalent_atoms

    # asking user list of correlated atoms and relative information:
    unique_df_list = []
    unique_corr_symbol_list = []
    unique_u_list = []
    unique_j_list = []
    unique_f_list = []
    unique_magmom_direction_list = []
    unique_nf_list = []
    for i, s in enumerate(material.symbols):
        if s in material.symbols[:i]:
            continue

        print('\n ' + '-'*12 + "\n atom {} {}".format(i,s))
        correlated = get_usr_input("\n Is this atom correlated?",
                ['y', 'n'])
        print(correlated, file=usr_input)
        if 'n' in correlated:
            continue
        unique_corr_symbol_list.append(s)

        df = get_usr_input_combo(
                "\n Enter correlated shells?", ['s', 'p', 'd', 'f'])
        print(df, file=usr_input)
        unique_df_list.append(df)

        if lhub in [1, 2]:
            while True:
                answer = input(
                        '\n Please provide interaction parameters U,J ' +
                        '\n separated by a space (eV): ')
                try:
                    answer = answer.split()
                    UJ = [float(answer[i]) for i in range(2)]
                    break
                except:
                    pass
            print(answer, file=usr_input)
            unique_u_list.append(UJ[0])
            unique_j_list.append(UJ[1])
            _l = "spdf".index(unique_df_list[-1])
            f_list = numpy.zeros(4)
            f_list[:_l+1] = U_J_to_radial_integrals(_l, UJ[0], UJ[1])
            unique_f_list.append(f_list)
        elif lhub == 3:
            while True:
                answer = input(
                        '\n Please provide radial integrals F0,F2,F4,F6' +
                        '\n separated by spaces and padded by 0s (eV): ')
                try:
                    answer = answer.split()
                    f_list = [float(answer[i]) for i in range(4)]
                    break
                except:
                    pass
            print(answer, file=usr_input)
            unique_f_list.append(f_list)
            _l = "spdf".index(unique_df_list[-1])
            U,J = radial_integrals_to_U_J(_l, f_list)
            unique_u_list.append(U)
            unique_j_list.append(J)
        if ldc == 2:
            while True:
                answer = input(
                        '\n Please provide the fixed number of' +
                        '\n localized {}-electrons for Vdc: '.format(df))
                try:
                    nf = float(answer)
                    break
                except:
                    continue
            print(answer, file=usr_input)
            unique_nf_list.append([nf/2,nf/2])

        if 'y' == spin_polarization == spin_orbit_coup:
            if 'y' == ferromagnetism:
                vec = ferro_magmom_direction
            else:
                vec = get_direction(log=usr_input)
        else:
            vec = [1., 0., 0.]
        unique_magmom_direction_list.append(vec)

    unique_corr_symbol_list = [s.encode() for s in unique_corr_symbol_list]
    f['/usrqa/unique_corr_symbol_list'] = unique_corr_symbol_list
    f['/usrqa/unique_df_list'] = [df.encode() for df in unique_df_list]
    if len(unique_u_list) > 0:
        f['/usrqa/unique_u_list_ev'] = unique_u_list
        f['/usrqa/unique_j_list_ev'] = unique_j_list
    if len(unique_f_list) > 0:
        f["/usrqa/unique_f_list_ev"] = unique_f_list
    if ldc == 2:
        f['/usrqa/unique_nf_list'] = unique_nf_list
    if 'y' == spin_polarization:
        f['/usrqa/unique_magmom_direction_list'] = \
                unique_magmom_direction_list

    if '-newton' in sys.argv:
        lnewton = sys.argv.index('-newton')+1
    else:
        print("\n Please select the method to solve G-RISB equations.\n" +
                " LNEWTON = 0: Recommended.\n" +
                "              Modified Powell hybrid method (HYDRD1).\n" +
                "          -1: Broyden method.")
        lnewton = get_usr_input(" Please select LNEWTON: ", ['-1', '0'])
        usr_input.write(lnewton + '\n')
    lnewton = int(lnewton)
    f['/usrqa/lnewton'] = lnewton

    if '-ed' in sys.argv:
        iembeddiag = sys.argv.index('-ed')+1
    else:
        print("\n Please select the method to solve embedding Hamiltonian.\n"+
                " LDIAG = -1: Valence truncation ED.\n" +
                "         -2: Valence truncation ED with Sz symmetry.\n" +
                "         -3: Valence truncation ED for S=0 (spin-singlet)\n"+
                "         -4: Valence truncation ED with Jz symmetry.\n" +
                "        -11: machine learning solver for soc only, exptl.\n"+
                "        -12: syten (dmrg) solver, exptl. \n" +
                "         10: HF (Mixing one-particle DM, exptl.).")
        iembeddiag = get_usr_input(" Please select LDIAG: ",  \
                ['-12', '-11', '-4', '-3', '-1', '-2', '10'])
        usr_input.write(iembeddiag + '\n')
    iembeddiag = int(iembeddiag)
    f['/usrqa/iembeddiag'] = iembeddiag
    usr_input.close()
    f.close()
    os.rename("input.slog", "init_ga.input")


def get_usr_input(message, accept_list):
    while True:
        answer = input(
                message +
                " \n Pick one from [" +
                ', '.join(item for item in accept_list) + "]...")
        if answer not in accept_list:
            print(" Please pick an answer in the list!" + \
                    " Make your choice again.")
        else:
            break
    return answer


def get_usr_input_combo(message, accept_list):
    while True:
        answer = input(
                message +
                " \n Pick one or combinations separated by blank space" +
                " \n from [" + ', '.join(item for item in accept_list) + "]...")
        if answer_valid(answer, accept_list):
            break
    return answer


def answer_valid(answer, accept_list):
    answer_list = answer.split()
    for ans in answer_list:
        if ans not in accept_list:
            return False
    return True


def get_direction(log=sys.stdout):
    '''get direction array(3).
    '''
    while True:
        vec_ = input(
                '\n please enter direction (to be normalized) \n'+\
                ' of the magnetic moment by components \n'+\
                ' in global coordinate system: x y z\n ')
        vec = vec_.split()
        vec = numpy.array(map(float, vec))
        if len(vec) == 3:
            vec = vec/numpy.linalg.norm(vec)
            break
        else:
            print(' enter 3 float numbers with finger space only.')
    print(vec_, file=log)
    return vec




from __future__ import print_function

import h5py
import numpy as np
from scipy.sparse import csr_matrix
from pygrisb.io.h5io import write_csr_matrix
from pygrisb.mbody.local_operator_factory import h5get_local_operators
from pygrisb.symm.atom_symm import get_representation


def h5get_hs_rotation_representation_N(imp, f_inp="init_ga_info.h5",
        f_log="glog.h5", valences=None):
    '''
    Calculate the Hilbert space rotation representation of each valence block
    of impurity imp in the hilbert_space_rotations.h5 file.
    '''

    f = h5py.File(f_inp, 'r')
    # 0-based in python
    Lie_Jeven_params = f["/impurity_" + str(imp - 1) + "/Lie_Jeven"][...]
    Lie_Jodd_params = f["/impurity_" + str(imp - 1) + "/Lie_Jodd"][...]
    f.close()

    f = h5py.File(f_log, 'r')
    op_dict = h5get_local_operators(imp, f, ["Jx", "Jy", "Jz"])

    rho_nrow = f["/Impurity_" + str(imp) + "/RHO.nrow"][0]
    # one-based to zero-based
    ID_N_block = f["/Impurity_" + str(imp) + "/Sec_ID"][...] - 1
    VAL_N_block = f["/Impurity_" + str(imp) + "/Sec_VAL"][...]
    f.close()
    VAL_N_block = [int(val + 0.5) for val in VAL_N_block]
    if valences is None:
        valences = VAL_N_block

    f = h5py.File("hilbert_space_rotations.h5", 'a')
    base = op_dict["Jx"].shape[0] - rho_nrow

    for i, val in enumerate(VAL_N_block):
        if ID_N_block[i + 1] - ID_N_block[i] <= 0:
            continue
        if val not in valences:
            continue
        Jx = op_dict["Jx"][base + ID_N_block[i] : base + ID_N_block[i + 1],
                           base + ID_N_block[i] : base + ID_N_block[i + 1]]
        Jy = op_dict["Jy"][base + ID_N_block[i] : base + ID_N_block[i + 1],
                           base + ID_N_block[i] : base + ID_N_block[i + 1]]
        Jz = op_dict["Jz"][base + ID_N_block[i] : base + ID_N_block[i + 1],
                           base + ID_N_block[i] : base + ID_N_block[i + 1]]
        if abs(np.mod(val, 2)) > 1.e-4:
            Rpr = get_representation([Jx, Jy, Jz], Lie_Jodd_params)
        else:
            Rpr = get_representation([Jx, Jy, Jz], Lie_Jeven_params)

        f["Impurity_{}/val_block={}/dim_rotations".format(imp, val)] = \
                len(Rpr)

        for j, _R in enumerate(Rpr):
            _R = csr_matrix(_R)
            write_csr_matrix(f,
                    "Impurity_{}/val_block={}/rotation_{}".format(imp, val, j),
                    _R)
    f.close()


if __name__ == "__main__":
    h5get_hs_rotation_representation_N(1, valences=[2])



import os, h5py, glob, sys
from math import pi, sqrt
import numpy as np
try:
    from builtins import zip
except:
    pass

from pygrisb.basic import units
import matplotlib.pyplot as plt


class DOS:

    def __init__(self, w_k, e_skn,  width=0.1, window=None, npts=201):
        """Electronic Density Of States object.

        width: float
          Width of guassian smearing.
        window: tuple of two float
          Use ``window=(emin, emax)``.  If not specified, a window
          big enough to hold all the eigenvalues will be used.
        npts: int
          Number of points.

        """

        self.npts = npts
        self.width = width
        self.w_k = w_k
        self.e_skn = e_skn

        if window is None:
            emin = self.e_skn.min() - 5 * self.width
            emax = self.e_skn.max() + 5 * self.width
        else:
            emin, emax = window

        self.energies = np.linspace(emin, emax, npts)


    def delta(self, energy):
        """
        Return a delta-function centered at energy.
        """
        x = -((self.energies - energy) / self.width)**2
        return np.exp(x) / (sqrt(pi) * self.width)


    def get_energies(self):
        """"
        return energy mesh.
        """
        return self.energies


    def get_dos_component(self, psi_skn_w):
        """
        Get array of DOS values.
        """
        dos_list = []

        # spin
        for e_kn, psi_kn_w in zip(self.e_skn,psi_skn_w):
            dos = np.zeros(self.npts, dtype=np.complex)
            # k-point
            for w, e_n, psi_n_w in zip(self.w_k, e_kn, psi_kn_w):
                # band index
                for e, psi_w in zip(e_n, psi_n_w):
                    dos += w * self.delta(e) * psi_w
            dos_list.append(dos)
        return np.array(dos_list)


    def get_dos_t(self):
        """
        Get array of DOS values.
        """
        dos_list = []

        # spin
        for e_kn in self.e_skn:
            dos = np.zeros(self.npts)
            # k-point
            for w, e_n in zip(self.w_k, e_kn):
                # band index
                for e in e_n:
                    dos += w * self.delta(e)
            dos_list.append(dos)
        return np.array(dos_list)


def get_bands(coherentonly=False):
    '''get band anergies and the correlated orbital characters.
    '''
    with h5py.File("GPARAMBANDS.h5", 'r') as f:
        # number of symmetry operations.
        nsym = f["/symnop"][0]
        # number of k-points
        nkp = f["/kptdim"][0]
        # maximal number of bands calculated over the k-points.
        nbmax = f["/nbmax"][0]
        # band index list specifying the range of bands used
        # for the construction of correlated orbitals.
        bnd_ne = []
        if "/NE_LIST_SPIN1" in f:
            bnd_ne.append(f["/NE_LIST_SPIN1"][()])
        elif "/NE_LIST" in f:
            bnd_ne.append(f["/NE_LIST"][()])
        else:
            bnd_ne.append([[nbmax, 1, nbmax] for k in range(nkp)])
        if "/NE_LIST_SPIN2" in f:
            bnd_ne.append(f["/NE_LIST_SPIN2"][()])

    with h5py.File("GLOG.h5", 'r') as f:
        # Gutzwiller fermi level
        e_fermi = f["/e_fermi"][0]
        # numbe of spin components
        nspin = f["/nspin"][0]
        # total number of correlated orbitals with spin included
        # for cases with spin-orbit interaction.
        nasotot = f["/nasotot"][0]
        if coherentonly:
            rmat = f["/BND_R"][()].swapaxes(1,2)

    if os.path.isfile('ginit.h5'):
        with h5py.File('ginit.h5', 'r') as f:
            use_rydberg = 'ryd' in f['/usrqa/unit'][()]
    else:
        use_rydberg = False

    # band energy array.
    e_skn = np.zeros((nspin,nkp,nbmax), dtype=np.float)

    # expansion coefficients of the correlated orbitals in terms of the
    # band wavefunctions, i.e., <\psi_{sks, n}|\phi_{sks, \alpha}>
    # with sks := ispin, ikpt, isym.
    if not coherentonly:
        psi_sksna = np.zeros((nspin,nkp,nsym,nbmax,nasotot),
                dtype=np.complex)
    else:
        psi_sksna = np.zeros((nspin,nkp,nsym,nbmax,nbmax),
                dtype=np.complex)

    # including the case with MPI run.
    for fname in glob.glob('GBANDS_*h5'):
        with h5py.File(fname, 'r') as f:
            for isp in range(nspin):
                for k in range(f['/IKP_START'][0]-1,f['/IKP_END'][0]):
                    e_n = f['/ISPIN_{}/IKP_{}/ek'.format(isp+1,k+1)][...]
                    e_n = e_n - e_fermi
                    # convert to eV
                    if use_rydberg:
                        e_n *= units.Ryd_eV
                    nbands = len(e_n)
                    e_skn[isp,k,:nbands] = e_n
                    # for bands not available, push it to high
                    # irrelevant value.
                    e_skn[isp,k,nbands:] = 100.
                    for isym in range(nsym):
                        if not coherentonly:
                            v = f['/ISPIN_{}/IKP_{}/ISYM_{}/EVEC'.format( \
                                    isp+1,k+1,isym+1)][:,:nasotot]
                            psi_sksna[isp,k,isym,bnd_ne[isp][k,1]-1: \
                                    bnd_ne[isp][k,1]+v.shape[0]-1,:] = v
                        else:
                            v = f['/ISPIN_{}/IKP_{}/ISYM_{}/EVEC'.format( \
                                    isp+1,k+1,isym+1)][()].T
                            v[:nasotot,:] = rmat[isp].T.conj().dot(
                                    v[:nasotot,:])
                            psi_sksna[isp,k,isym,bnd_ne[isp][k,1]-1: \
                                    bnd_ne[isp][k,1]+v.shape[1]-1, \
                                    bnd_ne[isp][k,1]-1: \
                                    bnd_ne[isp][k,1]+v.shape[1]-1] = v.T

    return e_skn, psi_sksna


def h5get_dos(ewin=(-3., 5.), delta=0.05, npts=1001, coherentonly=False):
    '''
    get total dos and the total correlated orbital component.
    '''
    with h5py.File("GPARAMBANDS.h5", 'r') as f:
        # list of k-point weight.
        w_k = f["/kptwt"][...]
    with h5py.File("GLOG.h5", 'r') as f:
        # total number of correlated orbitals with spin included
        # for cases with spin-orbit interaction.
        nasotot = f["/nasotot"][0]

    # get band energies and the orbital characters.
    e_skn, psi_sksna = get_bands(coherentonly=coherentonly)

    # get total dos
    dos = DOS(w_k, e_skn,  width=delta, window=ewin, npts=npts)
    energies = dos.get_energies()
    if not coherentonly:
        dos_t = dos.get_dos_t()
    else:
        # get coherent weight
        psi_skn_coh = np.einsum('...ijk,...ijk->...j', psi_sksna[...,:,:,:], \
                psi_sksna.conj()[...,:,:,:])/psi_sksna.shape[2]
        dos_t = dos.get_dos_component(psi_skn_coh)

    # get total correlated orbital component.
    psi_skn_f = np.einsum('...ijk,...ijk->...j', psi_sksna[...,:,:,:nasotot], \
            psi_sksna.conj()[...,:,:,:nasotot])/psi_sksna.shape[2]
    dos_f = dos.get_dos_component(psi_skn_f)

    return energies, dos_t, dos_f


def plot_dos_tf(energies, dos_t, dos_f):
    '''plot total dos and total correlated component
    and save in file `dos.pdf`.
    '''
    fig, ax = plt.subplots()
    if len(dos_t) == 1:
        ax.fill_between(
            energies, 0, dos_t[0], facecolor='grey', alpha=0.5)
        ax.plot(energies, dos_t[0], color='grey', label='tot')
        ax.plot(energies, dos_f[0], color='red', label='$corr. orb.$')
        ax.set_ylim(0.)
    else:
        ax.fill_between(
            energies, 0, dos_t[0], facecolor='grey', alpha=0.5)
        ax.fill_between(
            energies, 0, -dos_t[1], facecolor='grey', alpha=0.5)
        ax.plot(energies, dos_t[0], color='grey', label='tot-up')
        ax.plot(energies, -dos_t[1], color='grey', label='tot-dn')
        ax.plot(energies, dos_f[0], color='red', label='$corr-orb-up$')
        ax.plot(energies, -dos_f[1], color='blue', label='$corr-orb-dn$')
        ax.axhline(y=0, ls='-', color='black', linewidth=0.5)
    ax.set_xlim(np.min(energies), np.max(energies))
    ax.axvline(x=0, ls='--')
    ax.set_ylabel("DOS (states/f.u.)")
    ax.set_xlabel("E (eV)")
    plt.title("DOS with correlated orbital-component")
    plt.legend()
    fig.tight_layout()
    plt.show()
    fig.savefig('dos.pdf')


def driver_plot_dos():
    msg = r'''
    Script to plot total density of states with
    overall correlated orbital character.

    inline options:

        -el emin: set energy mesh minimuim
        -eh emax: set energy mesh miaximum
        -d delta: set Gaussian smearing factor
        -n nmesh: set energy mesh size
    '''

    emin = -5.0
    emax = 5.0
    delta = 0.05
    npts = 1001

    if '-h' in sys.argv:
        print(msg)
        sys.exit()
    else:
        if '-el' in sys.argv:
            emin = float(sys.argv[sys.argv.index('-el')+1])
        if '-eh' in sys.argv:
            emax = float(sys.argv[sys.argv.index('-eh')+1])
        if '-d' in sys.argv:
            delta = float(sys.argv[sys.argv.index('-d')+1])
        if '-n' in sys.argv:
            npts = int(sys.argv[sys.argv.index('-n')+1])
        if "-ch" in sys.argv:
            coherentonly = True
        else:
            coherentonly = False

    # get dos
    energies, dos_t, dos_f = h5get_dos(ewin=(emin, emax), delta=delta, \
            npts=npts, coherentonly=coherentonly)

    # pick up-component to plot
    plot_dos_tf(energies, dos_t, dos_f)


if __name__ == "__main__":
    '''
    Test run.
    '''
    energies, dos_t, dos_f = h5get_dos(ewin=(-8., 5.))
    # pick up-component.
    plot_dos_tf(energies, dos_t, dos_f)



import matplotlib.pyplot as plt


'''Simple pdf plot for quick visual check.
'''

colors = ['black', 'red', 'green', 'blue', 'orange','violet',
        'darkred', 'darkgreen', 'navy', 'brown', 'chocolate', 'darkorange',
        'gold', 'olive', 'maroon']

def xy_plot(x_list, y_list, xlabel='x', ylabel='y', fsave='test.pdf'):
    plt.figure()
    plt.plot(x_list, y_list)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.savefig(fsave)


def xy2_plot(x_list, y_list, pattern_list, label_list,
        xlabel='x', ylabel='y', fsave='test.pdf'):
    plt.figure()
    for x, y, pattern, label in zip(x_list, y_list, pattern_list, label_list):
        plt.plot(x, y, pattern, label=label)
    plt.legend()
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.savefig(fsave)



import itertools as it
import numpy, h5py, os
from pygrisb.gutz.atoms import Atoms
from pygrisb.basic.units import Ryd_eV
try:
    import spglib
except:
    import pyspglib.spglib as spglib


def cleanup_imap_list(imap_list):
    '''Clean up imap list in case correlated atoms are not well ordered
    in the struct file.
    '''
    imax = numpy.max(imap_list)+1
    index_map = numpy.zeros(imax, dtype=numpy.int)-1
    for i in range(imax):
        if i in imap_list:
            index_map[i] = imap_list.index(i)
    _imap_list = []
    for imap in imap_list:
        _imap_list.append(index_map[imap])
    return _imap_list


class gAtoms(Atoms):

    '''
    A class to describe the calculated system with all the relevant
    informations, such as coordinates, correlated atom indices,
    symmetry information, etc.
    '''
    def __init__(self, symbols=None, positions=None, numbers=None,
                 masses=None, magmoms=None, scaled_positions=None,
                 cell=None, pbc=None, wiencase=None, locrot_list=None):
        Atoms.__init__(self, symbols=symbols, positions=positions,
                numbers=numbers, masses=masses, magmoms=magmoms,
                scaled_positions=scaled_positions,
                cell=cell, pbc=pbc)
        self.wiencase = wiencase
        self.locrot_list = locrot_list

    def h5set(self):
        from pygrisb.io.h5io import h5auto_read
        with h5py.File('ginit.h5', 'r') as f:
            dist_cut = h5auto_read(f, \
                    '/usrqa/dist_cut', default=-1.0)
            self.set_sym_dist_cut(dist_cut)

            self.unit = h5auto_read(f, \
                    '/usrqa/unit', default='rydberg')

            spin_polarization = h5auto_read(f, \
                    '/usrqa/spin_polarization', default='n')
            self.set_ispin(spin_polarization)

            orbital_polarization = h5auto_read(f, \
                    '/usrqa/full_orbital_polarization', default='n')
            self.set_orbital_polarization(orbital_polarization)

            spin_orbit_coup = h5auto_read(f, \
                    '/usrqa/spin_orbit_coup', default='n')
            self.set_iso(spin_orbit_coup)

            crystal_field = h5auto_read(f, \
                    '/usrqa/crystal_field', default='y')
            self.set_crystal_field(crystal_field)

            lhub = h5auto_read(f, \
                    '/usrqa/u_matrix_type', default=1)
            self.set_lhub(lhub)

            ldc = h5auto_read(f, \
                    '/usrqa/ldc', default=1)
            self.set_ldc(ldc)

            idx_equivalent_atoms = h5auto_read(f, \
                    '/usrqa/idx_equivalent_atoms')
            idx_equivalent_atoms = idx_equivalent_atoms.tolist()
            self.set_idx_equivalent_atoms(idx_equivalent_atoms)

            unique_corr_symbol_list = h5auto_read(f, \
                    '/usrqa/unique_corr_symbol_list')
            self.unique_corr_symbol_list = [s.decode() for s in
                    unique_corr_symbol_list]
            self.unique_df_list = h5auto_read(f, \
                    '/usrqa/unique_df_list')
            self.unique_df_list = [df.decode() for df in self.unique_df_list]
            self.unique_nf_list = h5auto_read(f, '/usrqa/unique_nf_list')

            if self.unique_nf_list is not None:
                self.unique_nf_list = numpy.asfarray(self.unique_nf_list)
            if self.lhub in [1, 2]:
                self.unique_u_list = numpy.asfarray(h5auto_read(f, \
                        '/usrqa/unique_u_list_ev'))
                self.unique_j_list = numpy.asfarray(h5auto_read(f, \
                        '/usrqa/unique_j_list_ev'))
                if 'ryd' in self.unit:
                    self.unique_u_list /= Ryd_eV
                    self.unique_j_list /= Ryd_eV
            elif self.lhub == 3:
                self.unique_f_list = numpy.asfarray(h5auto_read(f, \
                        '/usrqa/unique_f_list_ev'))

            self.set_nval_range_list()
            self.set_CorrAtoms()
            self.set_na2_list()

            lnewton = h5auto_read(f, \
                    '/usrqa/lnewton', default=0)
            self.set_gimix(lnewton)

            iembeddiag = h5auto_read(f, \
                    '/usrqa/iembeddiag', default=-1)
            self.set_giembeddiag(iembeddiag)

            ferromagnetism = h5auto_read(f, \
                    "/usrqa/ferromagnetism", default="n")
            if "y" == ferromagnetism:
                self.fm_direction = \
                        f["/usrqa/unique_magmom_direction_list"][0]
            else:
                self.fm_direction = None


    # corr_list = list of indeces of correlated atoms listed in self.symbols
    def set_CorrAtoms(self):
        corr_list = []
        ityp_list = []
        imap_list = []
        df_list = []
        nf_list = []
        u_list = []
        j_list = []
        f_list = []
        for i, s in enumerate(self.symbols):
            if s in self.unique_corr_symbol_list:
                corr_list.append(i)
                ityp_list.append(self.unique_corr_symbol_list.index(s))
                df_list.append(self.unique_df_list[ityp_list[-1]])
                if self.unique_nf_list is not None:
                    nf_list.append(self.unique_nf_list[ityp_list[-1]])
                if self.lhub in [1, 2]:
                    u_list.append(self.unique_u_list[ityp_list[-1]])
                    j_list.append(self.unique_j_list[ityp_list[-1]])
                elif self.lhub == 3:
                    f_list.append(self.unique_f_list[ityp_list[-1]])
                idx_equ = self.idx_equivalent_atoms[i]
                imap = self.idx_equivalent_atoms.index(idx_equ)
                imap_list.append(imap)
        self.corr_list = corr_list
        self.ityp_list = ityp_list
        self.imap_list = cleanup_imap_list(imap_list)
        self.df_list = df_list
        self.u_list = u_list
        self.j_list = j_list
        self.f_list = f_list

        if len(nf_list) == 0:
            nf_list = None
        if self.modify_mode:
            with h5py.File('GPARAM.h5', 'r') as f:
                if '/dc_nelf_list' in f:
                    self.nelf_list = f['/dc_nelf_list'][()]
                else:
                    self.nelf_list = nf_list
        else:
            self.nelf_list = nf_list


    def set_lhub(self, lhub):
        self.lhub = lhub


    def set_ldc(self, ldc):
        self.ldc= ldc


    def set_modify_mode(self):
        self.modify_mode = os.path.exists('GPARAM.h5')


    def set_orbital_polarization(self, orbital_polarization):
        self.orbital_polarization = orbital_polarization


    def set_crystal_field(self, crystal_field):
        self.crystal_field = crystal_field


    def set_ispin(self, spin_polarization):
        self.ispin = 2 if 'y' in spin_polarization else 1


    def set_iso(self, spin_orbit_coup):
        self.iso = 2 if 'y' in spin_orbit_coup else 1


    def set_imap_list(self, imap_list):
        self.imap_list = imap_list


    def set_gimix(self, gimix):
        self.gimix = gimix


    def set_giembeddiag(self, giembeddiag):
        self.giembeddiag = int(giembeddiag)


    def set_sym_dist_cut(self, dist_cut):
        self.sym_dist_cut = dist_cut


    def get_Rotations(self, iat, Nmax=10):
        symbols = self.symbols
        cell = self.cell
        scaled_positions = self.scaled_positions
        if self.locrot_list is not None:
            locrot = self.locrot_list[iat]
        else:
            locrot = None
        equivalent_indices = self.idx_equivalent_atoms

        from pygrisb.gutz.molecule import xtal_get_local_rot
        return xtal_get_local_rot(symbols, scaled_positions, cell, iat,
                self.sym_dist_cut, equivalent_indices=equivalent_indices,
                locrot=locrot, fm_direction=self.fm_direction, Nmax=Nmax)


    def get_EquivalentAtoms(self):
        # Getting general information about symmetry of the lattice
        if self.wiencase is not None:
            from pygrisb.dft.wien import get_equivalent_atom_indices
            idx_equivalent_atoms = get_equivalent_atom_indices(
                    self.wiencase)
        else:
            dataset = spglib.get_symmetry_dataset(self, symprec=1e-5)
            idx_equivalent_atoms = dataset['equivalent_atoms'].tolist()
        return idx_equivalent_atoms


    def set_idx_equivalent_atoms(self, idx_equivalent_atoms):
        self.idx_equivalent_atoms = idx_equivalent_atoms


    def get_llist(self):
        if not hasattr(self, 'l_list'):
            from pygrisb.symm.angular_momentum_1p import get_l_list_from_string
            l_list = []
            AM_list = self.df_list
            for i, s in enumerate(self.corr_list):
                l_sublist = get_l_list_from_string(AM_list[i])
                l_list.append(l_sublist)
            self.l_list = l_list
        return self.l_list


    def set_SL_vector_list(self):
        '''
        Set S(L)_vector_list in the working symmetry-adapted basis.
        '''
        l_list = self.get_llist()
        sx_list = []; sy_list = []; sz_list = []
        lx_list = []; ly_list = []; lz_list = []
        from pygrisb.symm.angular_momentum_1p import get_S_vector, \
                get_L_vector
        for i, _ls in enumerate(l_list):
            S_vector = get_S_vector(_ls)
            L_vector = get_L_vector(_ls, iso=2)
            utrans = self.utrans_list[i]
            for j, _S, _L in zip(it.count(), S_vector, L_vector):
                S_vector[j] = utrans.T.conj().dot(_S).dot(utrans)
                L_vector[j] = utrans.T.conj().dot(_L).dot(utrans)
            sx_list.append(S_vector[0])
            sy_list.append(S_vector[1])
            sz_list.append(S_vector[2])
            lx_list.append(L_vector[0])
            ly_list.append(L_vector[1])
            lz_list.append(L_vector[2])
        self.sx_list = sx_list
        self.sy_list = sy_list
        self.sz_list = sz_list
        self.lx_list = lx_list
        self.ly_list = ly_list
        self.lz_list = lz_list


    def set_SelfEnergy(self):
        '''
        Get the self-energy structure and the corresponding unitary
        transformation of the local basis.
        '''
        from pygrisb.gutz.self_energy import get_self_energy
        utrans_list = []
        sigma_list = []
        l_list = self.get_llist()
        rotations_list = []
        jgenerator_list = []
        idx_equivalent_atoms = self.idx_equivalent_atoms
        if self.modify_mode:
            f = h5py.File('GPARAM.h5', 'r')
        for i, l in enumerate(self.corr_list):
            if self.modify_mode:
                utrans_list.append(f['/IMPURITY_'+str(i+1)+ \
                        '/DB_TO_SAB'][()].T)
                sigma_list.append(f['/IMPURITY_'+str(i+1)+ \
                        '/SIGMA_STRUCT'][()].T)
                if 'y' in self.crystal_field and 'n' in \
                        self.orbital_polarization:
                    rotations_list.append(f['/IMPURITY_'+str(i+1)+ \
                            '/rotations'][()])
                    jgenerator_list.append(f['/IMPURITY_'+str(i+1)+ \
                            '/JGENERATOR'][()])
                    jgenerator_list[-1] = numpy.swapaxes(jgenerator_list[-1],1,2)
            else:
                if i > 0:
                    idx_equ = idx_equivalent_atoms[i]
                    if idx_equ in idx_equivalent_atoms[:i]:
                        imap = idx_equivalent_atoms.index(idx_equ)
                        utrans_list.append(utrans_list[imap])
                        sigma_list.append(sigma_list[imap])
                        if len(rotations_list) > 0:
                            rotations_list.append(rotations_list[imap])
                            jgenerator_list.append(jgenerator_list[imap])
                        continue
                if 'y' in self.crystal_field and \
                        'n' in self.orbital_polarization:
                    rotations = self.get_Rotations(l)
                    rotations_list.append(rotations)
                else:
                    rotations = None
                jgen, utrans, sigma = get_self_energy(l_list[i], self.ispin,
                        self.orbital_polarization,
                        self.crystal_field, self.iso,
                        rotations=rotations)
                utrans_list.append(utrans)
                sigma_list.append(sigma)
                if jgen is not None:
                    jgenerator_list.append(jgen)
        self.utrans_list = utrans_list
        self.sigma_list = sigma_list
        if len(rotations_list) > 0:
            self.rotations_list = rotations_list
            self.jgenerator_list = jgenerator_list
        else:
            self.rotations_list = None
            self.jgenerator_list = None


    def set_LieParameters(self):
        '''
        Set the Lie parameters of rotation operators for odd and even J.
        '''
        import pygrisb.symm.atom_symm as atsym
        Lie_Jeven_list = []
        Lie_Jodd_list = []
        if 'y' in self.crystal_field and 'n' in self.orbital_polarization:
            for i, l in enumerate(self.corr_list):
                rotations = self.get_Rotations(l)
                Lie = atsym.get_Lie_parameters(rotations, plus2pi=False)
                Lie_Jeven_list.append(Lie)
                Lie = atsym.get_Lie_parameters(rotations, plus2pi=True)
                Lie_Jodd_list.append(Lie)
            self.Lie_Jeven_list = Lie_Jeven_list
            self.Lie_Jodd_list = Lie_Jodd_list
        else:
            self.Lie_Jeven_list = None
            self.Lie_Jodd_list = None


    def set_one_particle_rotation_list(self):
        '''Setup the rotation matrix in the single-particle basis
        '''
        import pygrisb.symm.atom_symm as atsym
        if self.Lie_Jodd_list is not None:
            sp_rotations_list = []
            for J, lies in zip(self.jgenerator_list, self.Lie_Jodd_list):
                if self.iso == 1:
                    J_ = [J1[::2, ::2] for J1 in J]
                else:
                    J_ = J
                sp_rotations_ = atsym.get_representation(J_, lies)
                if self.iso == 1:
                    sp_rotations = []
                    for sp_r_ in sp_rotations_:
                        sp_r = numpy.zeros_like(J[0])
                        sp_r[::2, ::2] = sp_r_
                        sp_r[1::2, 1::2] = sp_r[::2, ::2]
                        sp_rotations.append(sp_r)
                else:
                    sp_rotations = sp_rotations_
                sp_rotations_list.append(sp_rotations)
            self.sp_rotations_list = sp_rotations_list
        else:
            self.sp_rotations_list = None


    def set_na2_list(self):
        l_list = self.get_llist()
        na2_list = [numpy.sum(2 * (2 * numpy.array(ls) + 1)) for ls in l_list]
        self.na2_list = na2_list


    def set_nval_range_list(self):
        self.nval_bot_list=[]
        self.nval_top_list=[]
        for s in self.unique_corr_symbol_list:
            self.nval_bot_list.append(nval_range_list[s][0])
            self.nval_top_list.append(nval_range_list[s][1])


    def set_v2e_list(self):
        from pygrisb.mbody.coulomb_matrix import get_v2e_list
        l_list = self.get_llist()
        self.v2e_list, self.u_avg_list, self.j_avg_list = \
                get_v2e_list(self.lhub, l_list, \
                self.imap_list, self.utrans_list, \
                u_list=self.u_list, j_list=self.j_list, \
                f_list=self.f_list)


def h5calc_save_lrot(gatm):
    '''calcuate the 3d rotation matrix representations
    in Hilbert space fro each impurity.
    for equivalent impurity, a link is set up instead of
    redundant calculations.
    the results are save in 'GLROT.h5'.
    '''
    if gatm.giembeddiag != -31:
        return
    from pygrisb.mbody.local_operator_factory import get_lrot_op
    if os.path.isfile('GLROT.h5'):
        return
    with h5py.File('GLROT.h5', 'w') as f:
        for i, imap in enumerate(gatm.imap_list):
            prefix = '/IMPURITY_{}'.format(i+1)
            if i > imap:
                f[prefix] = h5py.SoftLink(
                        '/IMPURITY_{}'.format(imap+1))
                continue
            elif i < imap:
                raise ValueError('(i={}) > (imap={})'.format(i, imap))
            else:
                orb = gatm.df_list[i]
                if orb == 'd':
                    nmax = 10
                elif orb == 'f':
                    nmax = 14
                else:
                    raise ValueError('not supported df={}!'.format(orb))
                # empty or full occupation excluded.
                lvec = [gatm.lx_list[i], gatm.ly_list[i], gatm.lz_list[i]]
                for ival in range(1, nmax):
                    lrots = get_lrot_op(lvec, gatm.Lie_Jeven_list[i],
                            l=orb, ival=ival)
                    for ir, lrot in enumerate(lrots):
                        f[prefix+'/valence_block_{}/ROT_{}'.format( \
                                ival, ir+1)] = lrot.T
                f[prefix+'/dim_rot'] = [len(gatm.Lie_Jeven_list[i])]



nval_range_list = {
    "H":[0,2],
    "Li":[0,2],
    "Na":[0,2],
    "Sc":[0,10],
    "Ti":[0,10],
    "V":[0,10],
    "Cr":[0,10],
    "Mn":[0,10],
    "Fe":[0,10],
    "Co":[0,10],
    "Ni":[0,10],
    "Cu":[0,10],
    "Y":[0,10],
    "Zr":[0,10],
    "Nb":[0,10],
    "Mo":[0,10],
    "Tc":[0,10],
    "Ru":[0,10],
    "Rh":[0,10],
    "Pd":[0,10],
    "Ag":[0,10],
    "Ce":[0,3],
    "Pr":[0,4],
    "Nd":[0,5],
    "Pm":[0,8],
    "Sm":[0,9],
    "Eu":[0,10],
    "Gd":[0,11],
    "Tb":[0,12],
    "Dy":[0,13],
    "Ho":[0,14],
    "Er":[1,14],
    "Tm":[2,14],
    "Yb":[3,14],
    "Hf":[0,10],
    "Ta":[0,10],
    "W":[0,10],
    "Re":[0,10],
    "Os":[0,10],
    "Ir":[0,10],
    "Pt":[0,10],
    "U":[0,4],
    "Np":[0,14],
    "Pu":[0,14],
    "Am":[0,14],
    "Cm":[0,14],
    "Bk":[0,14],
    "Cf":[0,14],
    "Es":[0,14],
    "Fm":[0,14],
    "Md":[0,14]
        }



import os, sys, glob, h5py, socket, shutil, time, re, \
        subprocess, warnings
import numpy as np
from collections import deque
from subprocess import Popen, PIPE
from pygrisb.io.fio import file_exists
import pygrisb.run.environ as env


def get_file_info(fname, unit, idmf, case, scratch, so, para, cmplx, _band,
        updn, dnup):
    '''help function to setup informations in def file.
    '''
    if 'in2' == fname:
        return [unit, "'{}.in2{}'".format(case, cmplx), "'old'",
                "'formatted'", 0]
    elif 'inso' == fname:
        return [unit, "'{}.inso'".format(case), "'unknown'", "'formatted'", 0]
    elif 'indmfl' == fname:
        return [unit, "'{}.indmfl'".format(case), "'old'", "'formatted'", 0]
    elif 'outputdmfupdn' == fname:
        return [unit, "'{}.outputdmf{}{}'".format(case, idmf, updn), \
                "'unknown'", "'formatted'", 0]
    elif 'in1c' == fname:
        return [unit, "'{}.in1c'".format(case), "'unknown'", "'formatted'", 0]
    elif 'vectorupdn' == fname:
        return [unit, "'{}/{}.vector{}{}{}'".format(scratch, case, so, \
                updn, para), "'unknown'","'unformatted'",9000]
    elif 'vectordnup' == fname:
        return [unit, "'{}/{}.vector{}{}{}'".format(scratch, case, so, \
                dnup, para), "'unknown'","'unformatted'",9000]
    elif 'klist' == fname:
        return [unit, "'{}.klist{}'".format(case, _band), "'old'", \
                "'formatted'", 0]
    elif 'kgen' == fname:
        return [unit, "'{}.kgen'".format(case), "'unknown'", "'formatted'", 0]
    elif 'vspupdn' == fname:
        return [unit, "'{}.vsp{}'".format(case, updn), "'old'", \
                "'formatted'", 0]
    elif 'vspdnup' == fname:
        return [unit, "'{}.vsp{}'".format(case, dnup), "'unknown'", \
                "'formatted'", 0]
    elif 'struct' == fname:
        return [unit, "'{}.struct'".format(case), "'old'", "'formatted'", 0]
    elif 'rotlm' == fname:
        return [unit, "'{}.rotlm'".format(case), "'unknown'", "'formatted'", 0]
    elif 'energysodum' == fname:
        if so == 'so':
            sodum = 'dum'
        else:
            sodum = dnup
        return [unit, "'{}.energy{}'".format(case, sodum), \
               "'unknown'", "'formatted'", 0]
    elif 'energyupdn' == fname:
        return [unit, "'{}.energy{}{}{}'".format(case, so, updn, para), \
                "'unknown'", "'formatted'", 0]
    elif 'energydnup' == fname:
        return [unit, "'{}.energy{}{}{}'".format(case, so, dnup, para), \
                "'unknown'", "'formatted'", 0]
    elif 'clmval' == fname:
        return [unit, "'{}.clmval{}'".format(case, updn), "'unknown'", \
                "'formatted'", 0]
    elif 'recprlist' == fname:
        return [unit, "'{}.recprlist'".format(case), "'unknown'", \
                "'formatted'", 9000]
    elif 'scf2updn' == fname:
        return [unit, "'{}.scf2{}'".format(case, updn), \
                "'unknown'", "'formatted'", 0]
    elif 'normupdn' == fname:
        if so == "so" and updn == "":
            _updn = "up"
        else:
            _updn = updn
        return [unit, "'{}.norm{}{}{}'".format(case, so, _updn, para), \
                "'unknown'", "'formatted'", 0]
    elif 'normdnup' == fname:
        return [unit, "'{}.norm{}{}{}'".format(case, so, dnup, para), \
                "'unknown'", "'formatted'", 0]
    else:
        raise ValueError('No matching file name {}!'.format(fname))


def fcreate_def_gwien(case, scratch='.', so='', para='', idmf='1', cmplx='',
        _band='', updn='', dnup='dn'):
    '''create gwien1/2.def file.
    '''
    fdef = open('gwien{}{}.def'.format(idmf,updn), 'w')
    if idmf == '1':
        fname_list = ['in2', 'inso', 'indmfl', 'outputdmfupdn', \
                'in1c', 'vectorupdn', 'vectordnup', 'klist', \
                'kgen', 'vspupdn', 'vspdnup', 'struct', \
                'rotlm', 'energydnup', 'energyupdn', 'normupdn', \
                'normdnup']
        unit_list = [3, 4, 5, 6, \
                7, 9, 10, 13, \
                14, 18, 19, 20, \
                22, 59, 60, 12, \
                11]
    elif idmf == '2':
        fname_list = ['in1c', 'inso', 'in2', 'outputdmfupdn', 'indmfl', \
                'clmval', 'vectorupdn', 'vectordnup', 'recprlist', 'kgen', \
                'vspupdn', 'struct', 'scf2updn', 'rotlm', 'energyupdn', \
                'normupdn', 'normdnup']
        unit_list = [3, 4, 5, 6, 7, \
                8, 9, 10, 13, 14, \
                18, 20, 21, 22, 30, \
                12, 11]

    for fname, unit in zip(fname_list, unit_list):
        fdef.write("{:3d}, {:<15s}, {:<10s}, {:<13s}, {:<4d}\n".format(\
                *get_file_info(fname, unit, idmf, case, scratch, so, \
                para, cmplx, _band, updn, dnup)))
    fdef.close()


def onestep(fday, case, exec_name, w_root, para="", so="", \
        band=None, updn=None):
    '''wien2k steps.
    '''
    time_start = time.strftime("%H:%M:%S")
    cmd = ['{}/x'.format(w_root), exec_name, '-f', case]
    if para != "":
        cmd.append(para)
    if band == '-band':
        cmd.append(band)
        if not os.path.isfile('EFLDA.INP'):
            shutil.copy2('EFLDA.OUT', 'EFLDA.INP')
    if updn in ["-up", "-dn"]:
        cmd.append(updn)
    if so == "so":
        cmd.extend(["-c", "-so"])

    print(' '.join(x for x in cmd))
    process = Popen(cmd, stdout=PIPE, stderr=PIPE)
    out, err = process.communicate()
    fday.write('>{:<10s} ({}) {}\n'.format(exec_name, time_start, out[:-1]))
    fday.flush()
    for f in glob.glob('{}.error*'.format(exec_name)):
        if os.path.getsize(f) > 0:
            print('error in {} from file: {}'.format(
                    f, open(f, 'r').readlines()))
            sys.exit(1)


def gonestep(fday, exec_name, mpi, updn="", \
        gpath=os.environ['WIEN_GUTZ_ROOT']):
    '''gwien1, CyGutz and gwien2 steps.
    '''
    time_start = time.strftime("%H:%M:%S")
    with open(':log', 'a') as f:
        f.write('{}>   {}\n'.format(time.strftime("%a %b %d %H:%M:%S %Z %Y"), \
                exec_name))

    cmd = ['/usr/bin/time']
    if mpi != '':
        cmd.extend(mpi)
    cmd.append('{}'.format(exec_name))
    if 'gwien' in exec_name:
        cmd.append('{}{}.def'.format(exec_name, updn))
    if 'CyGutz' in exec_name:
        cmd += ['-p', gpath]

    print(' '.join(x for x in cmd))
    process = Popen(cmd, stdout=PIPE, stderr=PIPE)
    out, err = process.communicate()
    with open('{}_info.out'.format(exec_name), 'wb') as f:
        f.write(out)
    fday.write('>{:<10s} ({}) {}\n'.format(exec_name, time_start, \
            err.splitlines()[-2]))
    fday.flush()
    for f in glob.glob('{}.error*'.format(exec_name)):
        if os.path.getsize(f) > 0:
            print('error in {} from file: {}'.format(
                    f, open(f, 'r').readlines()))
            sys.exit(1)


def get_file_content(fname):
    if os.path.exists(fname):
        data = '\n------- {} --------\n'.format(fname)
        with open(fname, 'r') as f:
            data += f.read()
        return data
    else:
        return ''


def scf(case, spinpol):
    # scf file content
    if spinpol:
        f_list = ['{}.scf{}'.format(case, i) for i in ['0', \
                '1up', '1dn', 'so', '2up', '2dn', \
                '1s', '2s', 'cup', 'cdn']]
    else:
        f_list = ['{}.scf{}'.format(case, i) for i in ['0', \
                '1', 'so', '2', '1s', '2s', 'c']]

    data = ''.join(get_file_content(f) for f in f_list)

    with open('{}.scf'.format(case), 'a') as f:
        f.write(data)

    # files saved for mixing.
    if spinpol:
        f_list = ['clmsum', 'vspup', 'vspdn', 'vnsup', 'vnsdn', 'vrespsum',
                'clmdn', 'clmup']
    else:
        f_list = ['clmsum', 'vsp', 'vns', 'vrespsum']

    for i in f_list:
        name = '{}.{}'.format(case, i)
        if file_exists(name):
            shutil.copy2(name, '{}_old'.format(name))


def scfm(case):
    f_scf = '{}.scfm'.format(case)
    data = get_file_content(f_scf)

    with open('{}.scf'.format(case), 'a') as f:
        f.write(data)


def diff(fday, case, mix_dc, avg_dc, gskip):
    e_que = deque([], 2)
    with open('{}.scf'.format(case), 'r')  as f:
        for line in f:
            if ':DIS' in line:
                d_rho = float(line.split()[-1])
            if ':ENE' in line:
                e_que.append(float(line.split()[-1]))
    if len(e_que) == 2:
        d_etot = np.abs(e_que[1] - e_que[0])
    else:
        d_etot = 0.0

    dcv_err = 0.
    if not gskip:
        with h5py.File("GPARAM.h5", 'a') as f:
            ldc = f["/dc_mode"][0]
            if os.path.isfile("GDC_NELF_OUT.h5"):
                with h5py.File("GDC_NELF_OUT.h5", 'r') as fp:
                    nelf_list_inp = fp["/dc_nelf_list_inp"][()]
                    nelf_list_out = fp["/dc_nelf_list_out"][()]
                nelf_diff_list = nelf_list_out - nelf_list_inp
                nelf_list_mix = nelf_list_inp + mix_dc*nelf_diff_list
                if avg_dc:
                    valup = np.sum(nelf_list_mix[:,0])/nelf_list_mix.shape[0]
                    valdn = np.sum(nelf_list_mix[:,1])/nelf_list_mix.shape[0]
                    nelf_list_mix = [[valup,valdn] for x in nelf_list_inp]
                if ldc == 12:
                    if avg_dc:
                        dcv_err = np.sum(nelf_diff_list)/len(nelf_list_mix)
                    else:
                        dcv_err = np.max(np.abs(nelf_diff_list))
                    if '/dc_nelf_list' in f:
                        f["/dc_nelf_list"][()] = nelf_list_mix
                    else:
                        f["/dc_nelf_list"] = nelf_list_mix

    fday.write(':ENERGY convergence: {}\n'.format(d_etot))
    fday.write(':CHARGE convergence: {}\n'.format(d_rho))
    fday.write(':VDC convergence: {}\n'.format(dcv_err))
    return d_rho, d_etot, dcv_err


def processes_convert(so,updn):
    if not file_exists('.processes'):
        print('.processes file not present. It must be a serial run.')
        return
    lines = open('.processes', "r").readlines()
    work = {}
    nkstart = 0
    for line in lines:
        data = line.split(':')
        if data[0].strip().isdigit():
            vecn = ["emmanuel" for i in range(6)]
            i, nkp, nprc = map(int,data[::2])
            if not so:
                fdef = open('{}lapw1_{}.def'.format(updn,i), 'r')
                for line in fdef:
                    data = line.split(',')
                    data0 = int(data[0])
                    if data0 == 10 or data0 == 11:
                        data0 = data0 % 10
                        m = re.search('.*[\'|\"](.*)_(\d+)', data[1])
                        assert m is not None, 'vector file to macth ' + \
                                ' lapw1.def not found!'
                        vecn[data0*2] = '{}_{}'.format(m.group(1), m.group(2))
                        vecn[data0*2+1] = '{}dn_{}'.format(m.group(1), \
                                 m.group(2))
                fdef.close()
            else:
                fdef = open('{}lapwso_{}.def'.format(updn,i), 'r')
                for line in fdef:
                    data = line.split(',')
                    if int(data[0])==42:
                        vecn[0]=data[1].split("'")[1]
                    elif int(data[0])==41:
                        vecn[1]=data[1].split("'")[1]
                    elif int(data[0])==52:
                        vecn[2]=data[1].split("'")[1]
                    elif int(data[0])==51:
                        vecn[3]=data[1].split("'")[1]
                    elif int(data[0])==46:
                        vecn[4]=data[1].split("'")[1]
                    elif int(data[0])==45:
                        vecn[5]=data[1].split("'")[1]
                fdef.close()

            if work.has_key(nprc):
                work[nprc].append((i, nkp, nkstart, vecn))
            else:
                work[nprc]=[(i, nkp, nkstart, vecn)]
            nkstart += nkp

    for prc in sorted(work.keys()):
        fo = open('_processes_{}'.format(prc-1), 'w')
        for (i, nkp, nkstart, vecn) in work[prc]:
            fo.write('{} {} {} "{}" "{}" "{}" "{}" "{}" "{}"\n'.format(\
                    i, nkp, nkstart, *vecn))


def create_gomp_file():
    '''
    Create GOMP.h5 file based on GMPI_X.h5 for openMP execution.
    '''
    with h5py.File('GMPI_0.h5', 'r') as f:
        num_procs = f["/nprocs"][0]
    nvec = 0
    kvec1 = []
    kvec2 = []
    for iproc in range(num_procs):
        with h5py.File('GMPI_' + str(iproc) + '.h5', 'r') as f:
            nvec += f["/nvec"][0]
            kvec = f["/KVEC"][()].T
            kvec1.append(kvec[0])
            if kvec.shape[1] == 2:
                kvec2.append(kvec[1])
    kvec = np.asarray(kvec1 + kvec2)

    with h5py.File('GOMP.h5', 'w') as f:
        f['/nvec'] = np.asarray([nvec])
        f['/KVEC'] = kvec.T


def run_gwien(nmaxiter=100, mix_dc=0.2, cc=1.e-3, ec=1.e-5, vc=1.e-2,
        startp='lapw0', endp='', band='', dos='',
        openmp=False, cygutz='CyGutz',
        pa_list=[], recycle_rl=True, avg_dc=True, spinpol=False,
        p_so=False, gskip=False):
    '''Driver for Wien2k + Gutzwiller-Slave-boson job.
    '''
    if '-s' in sys.argv:
        startp = sys.argv[sys.argv.index('-s') + 1]
    if '-e' in sys.argv:
        endp = sys.argv[sys.argv.index('-e') + 1]
    if '-cc' in sys.argv:
        cc = float(sys.argv[sys.argv.index('-cc') + 1])
    if '-ec' in sys.argv:
        ec = float(sys.argv[sys.argv.index('-ec') + 1])
    if '-vc' in sys.argv:
        vc = float(sys.argv[sys.argv.index('-vc') + 1])
    if '-n' in sys.argv:
        nmaxiter = int(sys.argv[sys.argv.index('-n') + 1])
    if '-omp' in sys.argv:
        openmp = True
        print('Using Open-MP instead of MPI of CyGutz.')
    if '-amix' in sys.argv:
        mix_dc = float(sys.argv[sys.argv.index('-amix') + 1])
    if '-band' in sys.argv:
        band ='-band'
    if '-dos' in sys.argv:
        dos = '-dos'
    if '-nrl' in sys.argv:
        recycle_rl = False
    if '-navg_dc' in sys.argv:
        avg_dc = False
    if "-sp" in sys.argv:
        spinpol = True
    if "-so" in sys.argv:
        p_so = True
    if "-dft" in sys.argv:
        gskip = True
    if band == '-band':
        _band = '_band'
        nmaxiter = 1
    else:
        _band = ''
    if band == '-band' or dos == '-dos':
        cygutz = 'CyGutzB'

    if len(sys.argv) > 1 and sys.argv[1] in ['-h', '--help']:
        help = '''
    The script is a wrapper to run Wien2k + Gutzwiller.
    It usually loops over the following steps:

        x lapw0   : computes LDA potential with current DFT+G-RISB charge
        x lapw1   : solves LDA eigenvalue equations
        [x lapwso]: second variational treatment of spin-orbit coupling
        x gwien1  : compute the local projector in the basis of DFT bands
        x cygutz  : solve the generic KS-Hubbard model using G-RISB
        x gwien2  : computes DFT+G-RISB valence charge
        x lcore   : computes DFT core charge
        x mixer   : mixes new charge density with the previous result

    The parameters with default values are as follows:

        name     default  inline-argument  help
        --------------------------------------------------------------------
        nmaxiter 100      -n 100           max charge mixing steps
        mix_dc   0.2      -amix            D.C. potential mxing param
        cc       1.e-3    -cc 1.e-3        charge density cutoff to exit
        ec       1.e-5    -ec 1.e-5        total energy cutoff to exit
        startp   'lapw0'  -s lapw0         start program
        endp     ''       -e ''            end program
        openmp   False    -omp             use openMP instead of openMPI
        rl       True     -nrl             start from previous GA solutions
        avg_dc   True     -navg_dc         average dc among atoms or not
        spinpol  False    -sp              spin-symmetry breaking at DFT level
        '''
        print(help)
        sys.exit(0)

    para = ''
    _para = ''
    if file_exists('.machines') :
        para = ' -p'
        _para = '_x'

    toclean = glob.glob('*.scf*') + glob.glob('*.error*') + \
            glob.glob('*.outputdmf?.*') + glob.glob('EMBED_HAMIL_RES*')
    for f in toclean:
        os.remove(f)

    struct_file = glob.glob('*.struct')
    if len(struct_file) != 1:
        raise ValueError('{} struct files present while only one must exist!'. \
                format(len(struct_file)))
    w_case = struct_file[0].split('.')[0]
    w_root = os.environ['WIENROOT']
    w_scratch = os.environ['SCRATCH']
    g_root = os.environ['WIEN_GUTZ_ROOT']

    # infomation file
    fday = open(w_case + '.dayfile', 'w')
    fday.write('Calculating {} in {} \non {} with PID {}\n'.format(\
            w_case, os.getcwd(), socket.gethostname(), os.getpid()))

    if os.path.isfile(w_case + '.inso') and \
            os.path.getsize(w_case + '.inso') > 0 and not p_so:
        msg = 'spin-orbit is off while {} file is present!'.format(\
                w_case + '.inso')+'\nappending -so to turn it on.'
        warnings.warn(msg)

    if p_so:
        so='so'
        cmplx = 'c'
    else:
        so = ''
        cmplx = ''

    # In addition, check in1c file
    if file_exists(w_case+'.in1c'):
        cmplx = 'c'

    f_mpi = 'mpi_prefix.dat'
    if os.path.isfile(f_mpi):
        with open(f_mpi, 'r') as f:
            mpi = f.readline().split()
        print('{} exists -- running in parallel mode.'.format(f_mpi))
        print(' '.join(x for x in mpi))
    else:
        if para != '':
            raise ValueError('missing mpi_prefix.dat with .machines present!')
        mpi = ''
        print('{} not available -- running in serial mode.'.format(f_mpi))

    if openmp:
        _mpi = ''
    else:
        _mpi = mpi

    if not gskip:
        p_list = ['gwien1', 'gwien2']
        p_list.append(cygutz)
        for pa in pa_list:
            if pa not in p_list:
                p_list.append(pa)
        for p in p_list:
            shutil.copy2(g_root+'/'+p, '.')

        # create gwien1/2.def files
        if spinpol:
            fcreate_def_gwien(w_case, scratch=w_scratch, \
                    so=so, para=_para, \
                    idmf='1', cmplx=cmplx, _band=_band, \
                    updn="up", dnup='dn')
            if not p_so:
                fcreate_def_gwien(w_case, scratch=w_scratch, \
                        so=so, para=_para,
                        idmf='1', cmplx=cmplx, _band=_band, \
                                updn="dn", dnup='up')

            fcreate_def_gwien(w_case, scratch=w_scratch, \
                    so=so, para=_para, \
                    idmf='2', cmplx=cmplx, _band=_band, \
                    updn="up", dnup='dn')
            fcreate_def_gwien(w_case, scratch=w_scratch, \
                    so=so, para=_para, \
                    idmf='2', cmplx=cmplx, _band=_band, \
                    updn="dn", dnup='up')
        else:
            fcreate_def_gwien(w_case, scratch=w_scratch, \
                    so=so, para=_para, \
                    idmf='1', cmplx=cmplx, _band=_band)
            fcreate_def_gwien(w_case, scratch=w_scratch, \
                    so=so, para=_para, \
                    idmf='2', cmplx=cmplx, _band=_band)

    # save SLURM_ related environment variables
    slurm_envs = env.get_env_dict(key="SLURM_")

    if nmaxiter > 0:
        if not os.path.isfile('{}.clmsum'.format(w_case)):
            err_msg = 'no {}.clmsum file found--necessary for lapw0!'.\
                    format(w_case)
            print(err_msg)
            fday.print(err_msg+'\n')
            sys.exit(1)
        for f in glob.glob('*.broyd*'):
            os.remove(f)

        fday.write('   start at {} with {} \n    1/{} to go.\n'.format(
                time.asctime(), startp, nmaxiter))

    # Major charge density loop
    for icycle in range(nmaxiter):

        # unset slurm environment variables for wien2k type run
        env.unset_environ(slurm_envs)

        if icycle > 0 or (startp in 'lapw0'):
            onestep(fday, w_case, 'lapw0', w_root, para=para)
        if icycle > 0 or (startp in 'lapw0 lapw1'):
            if spinpol:
                onestep(fday, w_case, 'lapw1', w_root, para=para, band=band, \
                        updn="-up")
                onestep(fday, w_case, 'lapw1', w_root, para=para, band=band, \
                        updn="-dn")
            else:
                onestep(fday, w_case, 'lapw1', w_root, para=para, band=band)
        if (icycle > 0 or (startp in 'lapw0 lapw1 lapwso')) and p_so:
            if spinpol:
                onestep(fday, w_case, 'lapwso', w_root, para=para, \
                        band=band, updn="-up")
            else:
                onestep(fday, w_case, 'lapwso', w_root, para=para, band=band)

        if icycle==0 and para != '':
            if spinpol:
                processes_convert(p_so, updn="up")
            else:
                processes_convert(p_so, updn="")

        #set slurm environment variables for mpi run
        env.set_environ(slurm_envs)

        if gskip:
            # run dft only
            if icycle > 0 or (startp in 'lapw0 lapw1 lapwso lapw2'):
                if spinpol:
                    onestep(fday, w_case, 'lapw2', w_root, para=para, \
                            updn="-up", so=so)
                    onestep(fday, w_case, 'lapw2', w_root, para=para, \
                            updn="-dn", so=so)
                else:
                    onestep(fday, w_case, 'lapw2', w_root, para=para, so=so)
        else:
            if icycle > 0 or (startp in 'lapw0 lapw1 lapwso gwien1'):
                if spinpol:
                    gonestep(fday, 'gwien1', mpi, updn="up")
                    if not p_so:
                        gonestep(fday, 'gwien1', mpi, updn="dn")
                else:
                    gonestep(fday, 'gwien1', mpi)
                if openmp:
                    create_gomp_file()
                elif os.path.isfile("GOMP.h5"):
                    os.remove("GOMP.h5")
            if endp == 'gwien1':
                sys.exit(0)

            if icycle > 0 or (startp in 'lapw0 lapw1 lapwso gwien1 CyGutz'):
                gonestep(fday, cygutz, _mpi)
            if band == '-band' or dos == '-dos':
                sys.exit(0)
            shutil.copy2('GUTZ.LOG', 'SAVE_GUTZ.LOG')
            if endp == 'CyGutz':
                sys.exit(0)
            if recycle_rl:
                shutil.copy2('WH_RL_OUT.h5', 'WH_RL_INP.h5')

            if spinpol:
                gonestep(fday, 'gwien2', mpi, updn="up")
                gonestep(fday, 'gwien2', mpi, updn="dn")
            else:
                gonestep(fday, 'gwien2', mpi)

            if endp == 'gwien2':
                sys.exit(0)


        # unset slurm environment variables for wien2k type run
        env.unset_environ(slurm_envs)

        if spinpol:
            onestep(fday, w_case, 'lcore', w_root, para='', updn="-up")
            onestep(fday, w_case, 'lcore', w_root, para='', updn="-dn")
        else:
            onestep(fday, w_case, 'lcore', w_root, para='')

        scf(w_case, spinpol)
        onestep(fday, w_case, 'mixer', w_root, para='')
        scfm(w_case)
        drho, dene, dvdc = diff(fday, w_case, mix_dc, avg_dc, gskip)

        if gskip:
            gerr = 0.
        else:
            with h5py.File('GLOG.h5', 'r') as f:
                gerr = f['/rl_maxerr'][0]

        print(('dc={:.1e}, cc={:.1e} -> {:.0e}, ec={:.1e} ' + \
                '-> {:.0e}, gc={:.1e} icycle={}').format(
                dvdc, drho, cc, dene, ec, gerr, icycle))
        if drho < cc and dene < ec and dvdc < vc:
            sys.exit(0)


def batch_init_ga(dir_template='./template'):
    '''Loop over all the directories to initialize CyGutz calculations
     -- actually, since the CyGutz input files remain the same for different
    volumes, it simply copy the input files in template directory to
    each folder.
    '''
    cwd = os.getcwd()+'/'
    for dname in glob.glob('V*'):
        os.chdir(dname+'/case')
        shutil.copy(cwd+'/'+dir_template+'/ginit.h5', './')
        shutil.copy(cwd+'/'+dir_template+'/GPARAM.h5', './')
        if os.path.isfile(cwd+'/'+dir_template+'/GESOLVER.h5'):
            shutil.copy(cwd+'/'+dir_template+'/GESOLVER.h5', './')
        shutil.copy(cwd+'/'+dir_template+'/case.indmfl', './')
        os.chdir(cwd)


def batch_init_mott(dir_template='./template'):
    '''Loop over all the directories to initialize CyGutz-Mott calculations
     -- actually, since the CyGutz input files remain the same for different
    volumes, it simply copy the input files in template directory to
    each folder.
    '''
    cwd = os.getcwd()+'/'
    for dname in glob.glob('V*'):
        os.chdir(dname+'/case')
        shutil.copy(cwd+'/'+dir_template+'/GMOTT.h5', './')
        os.chdir(cwd)


def batch_modify_ga_setup(args, nproc=1):
    '''Loop over all the directories to modify CyGutz set up file.
    '''
    cwd = os.getcwd()+'/'
    cmd = [os.environ['WIEN_GUTZ_ROOT']+'/switch_gparam.py'] + args
    if '-p' in sys.argv:
        nproc = int(sys.argv[sys.argv.index('-p')+1])
    for i,dname in enumerate(glob.glob('V*')):
        os.chdir(dname+'/case')
        proc = subprocess.Popen(cmd)
        os.chdir(cwd)
        if (i+1) % nproc == 0:
            proc.communicate()


def batch_job_slurm(u, j, dir_template='./template', dir_work='./'):
    '''copy template/job.slurm file to each working directory and submit jobs.
    '''
    cwd = os.getcwd()+'/'
    jname = 'u{}j{}'.format(u, j)

    # command to modify u,j
    args = ['-unique_u_ev', u, '-unique_j_ev', j]
    cmd = [os.environ['WIEN_GUTZ_ROOT']+'/switch_gparam.py'] + args

    if '-w' in sys.argv:
        dir_work = sys.argv[sys.argv.index('-w')+1]

    # command to submit job.
    cmd_s = ['qsub', './job.slurm']

    for dname in glob.glob('V*'):
        os.chdir(dname+'/case/'+dir_work)

        # get job.slurm file
        with open(cwd+'/'+dir_template+'/job.slurm', 'r') as fin:
            with open('./job.slurm', 'w') as fout:
                for line in fin:
                    fout.write(line.replace('VV', dname). \
                            replace('UJ', jname))
        # modify u,j
        proc = subprocess.Popen(cmd)
        proc.communicate()

        # submit
        proc = subprocess.Popen(cmd_s)
        os.chdir(cwd)


def run_ga(nproc=1):
    '''Loop over all the directories to run_ga using nproc processors.
    '''
    cmd = [os.environ['WIEN_GUTZ_ROOT']+'/run_ga.py']
    cwd = os.getcwd()+'/'
    if '-p' in sys.argv:
        nproc = int(sys.argv[sys.argv.index('-p')+1])
    for i,dname in enumerate(glob.glob('V*')):
        os.chdir(dname+'/case')
        proc = subprocess.Popen(cmd)
        os.chdir(cwd)
        if (i+1) % nproc == 0:
            proc.communicate()


def batch_gsave(sdir='ldag', args=['-f']):
    '''Loop over all the directories to save_lapw.
    '''
    cmd = [os.environ['WIEN_GUTZ_ROOT']+'/save_ldag', '-d'] + [sdir] + args
    cwd = os.getcwd()+'/'
    for dname in glob.glob('V*'):
        os.chdir(dname+'/case')
        subprocess.call(cmd)
        os.chdir(cwd)



if __name__=='__main__':
    fcreate_def_gwien('FeSb2', scratch='.', so='', para='',
            idmf='1', cmplx='', _band='', updn='', dnup='dn')



import numpy as np
import pygrisb.gutz.ginput as ginput
import pygrisb.model.tbASE as tb
import pygrisb.model.special as special


def gutz_model_setup(u=0.0, nmesh=5000):
    '''Set up calculations for semi-circular DOS with two-ghost orbitals.
    '''
    # semi-circular energy mesh
    sc = special.semicirular()
    e_list = sc.get_e_list_of_uniform_wt(nmesh=nmesh)

    a = tb.AtomsTB("N", [(0, 0, 0)], cell=(1, 1, 1))
    a.set_orbitals_spindeg(orbitals=[("p")])
    # set up hk_list
    hk_list = [np.array([ \
            [e*1.e-0, 0, 0.j    ],\
            [0      , e, 0      ], \
            [0      , 0, e*1.e-0]]) for e in e_list]
    aTB = tb.TB(a, hk_list=hk_list)

    # dummy k-point list
    kps = e_list
    num_k = len(kps)
    kps_wt = 1.0 / num_k * np.ones((num_k))
    if aTB.Atoms.spindeg:
        kps_wt *= 2
    num_e = 0.0
    num_band_max = 3

    # GPARAMBANDS.h5

    h1e_list = [np.zeros((num_band_max*2, num_band_max*2), dtype=np.complex)]
    ginput.save_gparambands(kps_wt, num_e, num_band_max, \
            ensemble=1, h1e_list=h1e_list)

    # GPARAM.h5
    # self-energy structure
    sigma = np.zeros((6,6), dtype=int)
    sigma[0::2, 0::2] = np.arange(1,10).reshape((3,3))
    sigma[1::2, 1::2] = sigma[0::2, 0::2]

    # Coulomb matrix
    v2e = np.zeros((6, 6, 6, 6), dtype=np.complex)
    v2e[2, 2, 2, 2] = v2e[2, 2, 3, 3] = v2e[3, 3, 2, 2] = v2e[3, 3, 3, 3] = u
    vdc2_list = np.array([[0, 0, -u/2, -u/2, 0, 0]])

    ginput.save_gparam(na2_list=[6], iembeddiag=-3, imix=0, sigma_list=[sigma],
            v2e_list=[v2e], nval_top_list=[6], vdc2_list=vdc2_list)

    # BAREHAM_0.h5
    aTB.save_bareham(kps)


if __name__=='__main__':
    gutz_model_setup(u=1.0, nmesh=5000)



#!/usr/bin/env python
from __future__ import print_function
import h5py, numpy, os
from pygrisb.math.matrix_util import sym_array
from pygrisb.gutz.usrqa import get_usr_input
from pygrisb.basic.units import Ryd_eV
import pygrisb.mbody.double_counting as dc


def get_b_field_list():
    '''get the list of magnetic field by q&a.
    '''
    with h5py.File('ginit.h5', 'r') as f:
        unit = f['/usrqa/unit'][()]
        unique_magmom_direction_list = \
                f["/usrqa/unique_magmom_direction_list"][()]
    if 'ryd' in unit:
        unit = Ryd_eV
    else:
        unit = 1.0

    with h5py.File('GPARAM.h5', 'r') as f:
        ispin = f["/ispin"][0]
        if ispin == 1:
            return None, None
        num_imp = f["/num_imp"][0]
        imap_imp = f['/IMAP_IMP'][()]

    print(' total {} impurities with equivalence indices \n {}'.format( \
            num_imp, imap_imp))
    bvec_list = []
    imp_unique = 0
    for imp in range(num_imp):
        if imap_imp[imp] == imp + 1:
            print('\n IMPURITY {}'.format(imp+1))
            vec = unique_magmom_direction_list[imp_unique]
            print("\n magnetic moment direction is {}".format(vec) +
                    "\n in global coordinate system.")
            bm = input(
                    '\n please enter the magnitude of local B-field:'+\
                    '\n in unit of eV per Bohr magneton: \n ')
            bm = float(bm)/unit
            bvec_list.append(bm*vec)
        else:
            bvec_list.append(bvec_list[imap_imp[imp] - 1])
    givext = get_usr_input( \
            '\n Is the external field applied only at initial step (0) \n'+ \
            ' or fixed through all the iterations (1).', ['0', '1'])
    givext = int(givext)
    return bvec_list, givext


def get_sym_2darray(a, imp=1):
    '''get symmetrized 2d array with error reporting.
    '''
    with h5py.File('GPARAM.h5', 'r') as f:
        path = '/IMPURITY_{}/SP_ROTATIONS'.format(imp)
        if path in f:
            r_list = numpy.swapaxes(f[path][()], 1, 2)
            a_sym = sym_array(a, r_list)
        else:
            a_sym = a
        sym_err = numpy.max(numpy.abs(a-a_sym))
    return a_sym, sym_err


def get_vext_list(bvec_list):
    '''get the list of magnetic potential matrix in the single particle space,
    given the list of local magnetic field.
    '''
    vext_list = []
    max_sym_err = 0.
    with h5py.File('GPARAM.h5', 'r') as f:
        for imp, bvec in enumerate(bvec_list):
            prepath = "/IMPURITY_" + str(imp + 1)
            sx = f[prepath+'/SX'][()].T
            sy = f[prepath+'/SY'][()].T
            sz = f[prepath+'/SZ'][()].T
            vext = (bvec[0]*sx + bvec[1]*sy + bvec[2]*sz)*2
            lx = f[prepath+'/LX'][()].T
            ly = f[prepath+'/LY'][()].T
            lz = f[prepath+'/LZ'][()].T
            vext += bvec[0]*lx + bvec[1]*ly + bvec[2]*lz
            vext_sym, sym_err = get_sym_2darray(vext, imp+1)
            max_sym_err = max(max_sym_err, sym_err)
            vext_list.append(vext_sym)
    if max_sym_err > 1.e-5:
        print(" Warning:")
    print(' maximal symmetrization error of vext = {}'.format(max_sym_err))
    return vext_list


def get_vext_given_1pdm_list(dm_list):
    '''get the external potential for lda+u calculation
    given the initial one-particle density matrix.
    '''
    with h5py.File("GPARAM.h5", "r") as f:
        javg_list = f["/dc_j_avg"][()]
        uavg_list = f["/dc_u_avg"][()]
        v2e_list = []
        for i in range(f["/num_imp"][0]):
            v2e_list.append(f["/IMPURITY_{}/V2E".format(i+1)][()].T)

    vext_list = [-dc.get_vdc_hf(v2e, dm)+ \
            dc.get_vdc_fll(uavg, javg, numpy.trace(dm))*\
            numpy.eye(dm.shape[0]) \
            for v2e, dm, uavg, javg in zip(v2e_list, dm_list, \
            uavg_list, javg_list)]
    return vext_list


def chk_local_one_body(vext_list):
    if not os.path.isfile("GPARAMBANDS.h5"):
        return
    db2sab_list = []
    sx_list = []
    sy_list = []
    sz_list = []
    lx_list = []
    ly_list = []
    lz_list = []
    with h5py.File('GPARAM.h5', 'r') as f:
        num_imp = f["/num_imp"][0]
        for imp in range(num_imp):
            prepath = "/IMPURITY_" + str(imp + 1)
            db2sab_list.append(f[prepath+"/DB_TO_SAB".format(imp+1)] \
                    [()].T)
            sx_list.append(f[prepath+"/SX"][()].T)
            sy_list.append(f[prepath+"/SY"][()].T)
            sz_list.append(f[prepath+"/SZ"][()].T)
            lx_list.append(f[prepath+"/LX"][()].T)
            ly_list.append(f[prepath+"/LY"][()].T)
            lz_list.append(f[prepath+"/LZ"][()].T)
    h1e_list = []
    with h5py.File("GPARAMBANDS.h5", 'r') as f:
        for imp in range(num_imp):
            h1e = f["/IMPURITY_{}/H1E".format(imp+1)][()].T
            db2sab = db2sab_list[imp]
            h1e = db2sab.T.conj().dot(h1e).dot(db2sab)
            h1e_list.append(h1e)
    # start actually analysis
    numpy.set_printoptions(precision=2, suppress=True)
    for imp in range(num_imp):
        h = h1e_list[imp] + vext_list[imp]
        w, v = numpy.linalg.eigh(h)
        dm = v[:,0:1].dot(v[:,0:1].T.conj())
        dm_sym, _ = get_sym_2darray(dm, imp+1)
        print(" <Sx> = {}".format(numpy.sum(dm_sym*sx_list[imp])))
        print(" <Sy> = {}".format(numpy.sum(dm_sym*sy_list[imp])))
        print(" <Sz> = {}".format(numpy.sum(dm_sym*sz_list[imp])))
        print(" <Lx> = {}".format(numpy.sum(dm_sym*lx_list[imp])))
        print(" <Ly> = {}".format(numpy.sum(dm_sym*ly_list[imp])))
        print(" <Lz> = {}".format(numpy.sum(dm_sym*lz_list[imp])))


def h5wrt_gmagnet(vext_list, g_ivext, fn='GVEXT.h5'):
    with h5py.File(fn, 'w') as f:
        for imp, vext in enumerate(vext_list):
            f['/IMPURITY_{}/VEXT'.format(imp+1)] = vext.T
        f['/givext'] = [g_ivext]


def init_magnet_conf():
    '''
    initialize the the magnetic configuration for magnetic calculation.
    '''
    bvec_list, g_ivext = get_b_field_list()
    if bvec_list is None:
        return
    vext_list = get_vext_list(bvec_list)
    chk_local_one_body(vext_list)
    h5wrt_gmagnet(vext_list, g_ivext)


def init_magnet_conf_with_init_dm(dm_list):
    '''
    initialize the the magnetic configuration for magnetic calculation
    based on given initial one-particle density-matrix.
    '''
    vext_list = get_vext_given_1pdm_list(dm_list)
    chk_local_one_body(vext_list)
    h5wrt_gmagnet(vext_list, g_ivext=0)


if __name__ == "__main__":
    init_magnet_conf()



import numpy as np


def cart2sph(v):
    '''converto v from cartesian coordinates to spherical coordinates.
    '''
    r = np.linalg.norm(v)
    abmax = np.max(np.abs(v[:2]))
    if abmax < 1.e-16:
        theta = 0.
        # take an arbitary value.
        phi = 0.
    else:
        v /= abmax
        ab = np.sqrt(v[0]**2+v[1]**2)
        phi = np.arcsin(v[1]/ab)
        if v[0] < 0.:
            phi = np.pi - phi
        theta = np.arccos(v[2]/np.linalg.norm(v))
    return r, theta, phi



from __future__ import print_function
import numpy as np

try:
    from builtins import range, zip
except:
    pass


def fermi_dirac(e_fermi, delta, energy):
    """
    Return fermi-dirac distribution weight.
    """
    x = (energy - e_fermi)/delta
    if x < -200:
        f = 1.
    elif x > 200:
        f = 0.
    else:
        f = 1./(np.exp(x) + 1)
    return f


def num_electron_diff(e_fermi, delta, e_skn, w_k, nb_k, num_elec):
    ne = 0
    for e_kn in e_skn:
        for e_n, w, nb in zip(e_kn, w_k, nb_k):
            f = [fermi_dirac(e_fermi, delta, e) for e in e_n[:nb]]
            ne += np.sum(f)*w
    return ne - num_elec



import os, sys, subprocess
from pygrisb.gutz import embedh
from pygrisb.gsolver.gs_idmrg import create_idmrg_ctl_file


def driver_ridmrg(imp=1, lv2e=False):
    '''dmrg solver based on ITensor.
    '''
    if "-i" in sys.argv:
        imp = int(sys.argv[sys.argv.index("-i")+1])
    h1e, lambdac, daalpha, v2e = embedh.h5gen_embedh_spin_updn(imp=imp,
            lv2e=lv2e)
    embedh.wrt_text_rembed(h1e, lambdac, daalpha, v2e, shreshold=1.e-7)
    N = Npart = h1e.shape[0]
    create_idmrg_ctl_file(imp=imp, N=N, Npart=Npart)
    cmd = [os.environ["WIEN_GUTZ_ROOT2"]+"/exe_idmrg", str(imp)]
    with open("GDMRG_{}.LOG".format(imp), "w") as f:
        subprocess.call(cmd, stdout=f)
    dm, e_tot = embedh.get_res_idmrg(imp=imp)
    embedh.h5wrt_dm_sab_rc(dm, e_tot, imp=imp)


if __name__ == "__main__":
    driver_ridmrg(lv2e=True)



import numpy as np
from pygrisb.model import circauxi
import shutil,subprocess
import h5py

mu_list = [1.569]+ \
        np.arange(1.57,1.6,0.01).tolist() + np.arange(1.6,3.0,0.1).tolist()
cmd = ['/home/ykent/WIEN_GUTZ/bin2/CyGutz', '-r', '-1']
e_list = []
n_list = []
u = 5.0
frec = open('mu_u5.0.dat', 'w')
for mu in mu_list:
    circauxi.gutz_model_setup(u=u, nmesh=5000, norb=3, tiny=0.0, mu=mu)
    shutil.copyfile('U{0}/WH_RL_INIT.h5_u{0}mu{1}'.format(u,mu),'WH_RL_INIT.h5')
    subprocess.call(cmd)
    shutil.copyfile('WH_RL_BEST.h5', 'WH_RL_INIT.h5_u{}mu{}'.format(u,mu))

    with h5py.File('EMBED_HAMIL_RES_1.h5', 'r') as f:
        n = f['/DM'][0,0] + f['/DM'][1,1]
        n_list.append(n)
    with h5py.File('GLOG.h5', 'r') as f:
        e = f['/etot_model'][0] + u/2*n
        e_list.append(e)

    frec.write('{} {} {}\n'.format(mu, e.real, n.real))
    frec.flush()
frec.close()



######################################
# some double counting potential form.
######################################
import numpy


def get_vdc_hf(v2e, dm):
    '''get Hartree-Fock type double counting potential,
    given the full coulomb matrix (chemist convention, including spin index)
    and the one-particle density matrix.
    The usual minus sign is included.
    '''
    vdc = -numpy.einsum('ijkl,kl->ij', v2e, dm)
    vdc += numpy.einsum('ikjl,kl->ij', v2e, dm)
    return vdc


def get_vdc_fll(uavg, javg, ntot):
    '''get double counting potential in the full-localization limit.
    The usual minus sign is included.
    '''
    vdc = -uavg*(ntot-0.5) + javg/2*(ntot-1)
    return vdc


def get_vdc_amf(uavg, javg, ntot, l):
    '''get double counting potential in the around mean-field form.
    The usual minus sign is included.
    '''
    alpha = -uavg/2.+(uavg+2*l*javg)/(2*l+1)/4.
    vdc = alpha*2*ntot
    return vdc



if __name__ == "__main__":
    uavg, javg, ntot, l = 6.0, 0.8, 5.2, 3
    print(get_vdc_fll(uavg, javg, ntot))
    print(get_vdc_amf(uavg, javg, ntot, l))
